import tempfile
import unittest
from typing import Dict

import yaml

from tira.io_utils import patch_ir_metadata


def patch(original_dict: Dict, src_pattern: Dict, target_pattern: Dict):
    with tempfile.TemporaryDirectory() as tmp_dir:
        additional_fields = ["platform", "implementation", "resources"]
        ret = str(tmp_dir) + "/foo.yml"
        original_dict = original_dict.copy()
        for f in additional_fields:
            original_dict[f] = ""
        yaml.safe_dump(original_dict, open(ret, "w"))
        patch_ir_metadata(str(tmp_dir), src_pattern, target_pattern)
        ret = yaml.safe_load(open(ret))
        for f in additional_fields:
            del ret[f]
        return ret


class TestPatchIrMetadata(unittest.TestCase):
    def test_does_nothing_01(self):
        actual = patch(
            {"hello": "world"},
            {"does-not-exist": "a"},
            {"does-not-exist": "b"},
        )
        self.assertEqual({"hello": "world"}, actual)

    def test_does_nothing_02(self):
        src_pattern = {"data": {"test collection": {"name": "foo-1"}}}
        target_pattern = {"data": {"test collection": {"name": "foo-2"}}}

        actual = patch({"hello": "world"}, src_pattern, target_pattern)
        self.assertEqual({"hello": "world"}, actual)

    def test_does_nothing_03(self):
        src_pattern = {"data": {"test collection": {"name": "foo-1"}}}
        target_pattern = {"data": {"test collection": {"name": "foo-2"}}}

        actual = patch(
            {"hello": "world", "data": {"test collection": {"name": "some-test-collection"}}},
            src_pattern,
            target_pattern,
        )
        self.assertEqual(
            {
                "hello": "world",
                "data": {"test collection": {"name": "some-test-collection"}},
            },
            actual,
        )

    def test_patch_works(self):
        src_pattern = {"data": {"test collection": {"name": "foo-1"}}}
        target_pattern = {"data": {"test collection": {"name": "foo-2"}}}

        actual = patch(
            {"hello": "world", "data": {"test collection": {"name": "foo-1"}}},
            src_pattern,
            target_pattern,
        )
        self.assertEqual(
            {
                "hello": "world",
                "data": {"test collection": {"name": "foo-2"}},
            },
            actual,
        )



import os
import unittest
from pathlib import Path

from tira.io_utils import _ln_huggingface_model_mounts, huggingface_model_mounts


class MountHfModelTest(unittest.TestCase):
    def test_none_returns_empty_dict(self):
        expected = {}
        actual = huggingface_model_mounts(None)
        self.assertEqual(actual, expected)

    def test_empty_list_returns_empty_dict(self):
        expected = {}
        actual = huggingface_model_mounts([])
        self.assertEqual(actual, expected)

    def test_exception_is_thrown_for_non_existing_model(self):
        with self.assertRaises(Exception) as context:
            huggingface_model_mounts(["model-does-not-exist"])
        self.assertTrue("model-does-not-exist" in str(context.exception))

    def test_mounts_for_single_existing_model(self):
        expected = {
            "models--openai-community--gpt2": {
                "bind": "/root/.cache/huggingface/hub/models--openai-community--gpt2",
                "mode": "ro",
            }
        }
        os.environ["HF_HUB_CACHE"] = "tests/resources/hf/hub"
        actual = huggingface_model_mounts(["openai-community/gpt2"])
        actual = {str(k).split("/")[-1]: v for k, v in actual.items()}
        del os.environ["HF_HUB_CACHE"]
        self.assertEqual(actual, expected)

    def test_mounts_for_multiple_existing_model(self):
        expected = {
            "models--openai-community--gpt2": {
                "bind": "/root/.cache/huggingface/hub/models--openai-community--gpt2",
                "mode": "ro",
            },
            "models--openai-community--gpt2-large": {
                "bind": "/root/.cache/huggingface/hub/models--openai-community--gpt2-large",
                "mode": "ro",
            },
        }
        os.environ["HF_HUB_CACHE"] = "tests/resources/hf/hub"
        actual = huggingface_model_mounts(["openai-community/gpt2", "openai-community/gpt2-large"])
        actual = {str(k).split("/")[-1]: v for k, v in actual.items()}
        del os.environ["HF_HUB_CACHE"]
        self.assertEqual(actual, expected)

    def test_mounts_for_multiple_redundant_existing_model(self):
        expected = {
            "models--openai-community--gpt2": {
                "bind": "/root/.cache/huggingface/hub/models--openai-community--gpt2",
                "mode": "ro",
            },
            "models--openai-community--gpt2-large": {
                "bind": "/root/.cache/huggingface/hub/models--openai-community--gpt2-large",
                "mode": "ro",
            },
        }
        os.environ["HF_HUB_CACHE"] = "tests/resources/hf/hub"
        actual = huggingface_model_mounts(
            [
                "openai-community/gpt2",
                "openai-community/gpt2-large",
                "openai-community/gpt2",
                "openai-community/gpt2-large",
            ]
        )
        actual = {str(k).split("/")[-1]: v for k, v in actual.items()}
        del os.environ["HF_HUB_CACHE"]
        self.assertEqual(actual, expected)

    def test_ln_mount_for_none(self):
        expected = ""
        actual = _ln_huggingface_model_mounts(None)
        self.assertEqual(actual, expected)

    def test_ln_mount_for_empty_string(self):
        expected = ""
        actual = _ln_huggingface_model_mounts("")
        self.assertEqual(actual, expected)

    def test_ln_mount_for_empty_string_after_strip(self):
        expected = ""
        actual = _ln_huggingface_model_mounts("    ")
        self.assertEqual(actual, expected)

    def test_ln_mount_for_non_existing_model_throwns_exception(self):
        with self.assertRaises(Exception) as context:
            _ln_huggingface_model_mounts("this-model-does-not-exist")
        self.assertTrue("this-model-does-not-exist" in str(context.exception))

    def test_ln_mounts_for_single_existing_model(self):
        expected = (
            "mkdir -p /root/.cache/huggingface/hub/; rm -Rf"
            " /root/.cache/huggingface/hub/models--openai-community--gpt2; ln -s"
            " <NORMALIZED>/hub/models--openai-community--gpt2"
            ' /root/.cache/huggingface/hub/models--openai-community--gpt2; echo "mounted 1 models"'
        )
        os.environ["HF_HUB_CACHE"] = "tests/resources/hf/hub"
        actual = _ln_huggingface_model_mounts("openai-community/gpt2")
        actual = actual.replace(str(Path("tests/resources/hf").absolute()), "<NORMALIZED>")
        del os.environ["HF_HUB_CACHE"]
        self.assertEqual(actual, expected)

    def test_ln_mounts_for_single_existing_model_redundant(self):
        expected = (
            "mkdir -p /root/.cache/huggingface/hub/; rm -Rf"
            " /root/.cache/huggingface/hub/models--openai-community--gpt2; ln -s"
            " <NORMALIZED>/hub/models--openai-community--gpt2"
            ' /root/.cache/huggingface/hub/models--openai-community--gpt2; echo "mounted 1 models"'
        )
        os.environ["HF_HUB_CACHE"] = "tests/resources/hf/hub"
        actual = _ln_huggingface_model_mounts("openai-community/gpt2 openai-community/gpt2   openai-community/gpt2")
        actual = actual.replace(str(Path("tests/resources/hf").absolute()), "<NORMALIZED>")
        del os.environ["HF_HUB_CACHE"]
        self.assertEqual(actual, expected)

    def test_ln_mounts_for_multiple_existing_models(self):
        expected = (
            "mkdir -p /root/.cache/huggingface/hub/; rm -Rf"
            " /root/.cache/huggingface/hub/models--openai-community--gpt2; ln -s"
            " <NORMALIZED>/hub/models--openai-community--gpt2"
            " /root/.cache/huggingface/hub/models--openai-community--gpt2; rm -Rf"
            " /root/.cache/huggingface/hub/models--openai-community--gpt2-large; ln -s"
            " <NORMALIZED>/hub/models--openai-community--gpt2-large"
            ' /root/.cache/huggingface/hub/models--openai-community--gpt2-large; echo "mounted 2 models"'
        )
        os.environ["HF_HUB_CACHE"] = "tests/resources/hf/hub"
        actual = _ln_huggingface_model_mounts("openai-community/gpt2 openai-community/gpt2-large")
        actual = actual.replace(str(Path("tests/resources/hf").absolute()), "<NORMALIZED>")
        del os.environ["HF_HUB_CACHE"]
        self.assertEqual(actual, expected)

    def test_ln_mounts_for_multiple_existing_models_redundant(self):
        expected = (
            "mkdir -p /root/.cache/huggingface/hub/; rm -Rf"
            " /root/.cache/huggingface/hub/models--openai-community--gpt2; ln -s"
            " <NORMALIZED>/hub/models--openai-community--gpt2"
            " /root/.cache/huggingface/hub/models--openai-community--gpt2; rm -Rf"
            " /root/.cache/huggingface/hub/models--openai-community--gpt2-large; ln -s"
            " <NORMALIZED>/hub/models--openai-community--gpt2-large"
            ' /root/.cache/huggingface/hub/models--openai-community--gpt2-large; echo "mounted 2 models"'
        )
        os.environ["HF_HUB_CACHE"] = "tests/resources/hf/hub"
        actual = _ln_huggingface_model_mounts(
            "openai-community/gpt2 openai-community/gpt2-large openai-community/gpt2 openai-community/gpt2-large"
            " openai-community/gpt2 openai-community/gpt2-large"
        )
        actual = actual.replace(str(Path("tests/resources/hf").absolute()), "<NORMALIZED>")
        del os.environ["HF_HUB_CACHE"]
        self.assertEqual(actual, expected)



import tempfile
import unittest
from pathlib import Path

from tira.check_format import check_format, lines_if_valid

from . import _ERROR, _OK, EMPTY_OUTPUT, IR_QUERY_OUTPUT, VALID_RUN_OUTPUT

EXAMPLE_01 = """
run-01 relstring             	979	'----------'
run-01 recall_5              	979	0.1
run-01 recall_10             	979	0.2
run-01 recall_15             	979	0.3
run-01 relstring             	all	'----------'
run-01 recall_5              	all	0.1
run-01 recall_10             	all	0.2
run-01 recall_15             	all	0.3
"""

EXAMPLE_02 = """
run-02 relstring             	979	'----------'
run-02 recall_5              	979	0.3
run-02 recall_10             	979	0.4
run-02 recall_15             	979	0.6
run-02 relstring             	all	'----------'
run-02 recall_5              	all	0.7
run-02 recall_10             	all	0.8
run-02 recall_15             	all	0.9
"""


class TestTrecEvalLeaderbaordFormat(unittest.TestCase):
    def test_invalid_validator_on_empty_output(self):
        expected = [
            _ERROR,
            "No unique file in trec-eval -q format was found, only the files ['.gitkeep'] were available.",
        ]
        actual = check_format(EMPTY_OUTPUT, "trec-eval-leaderboard")
        self.assertEqual(expected, actual)

    def test_invalid_validator_ir_query_output(self):
        expected = [
            _ERROR,
            "No unique file in trec-eval -q format was found, only the files ['queries.jsonl'] were available.",
        ]
        actual = check_format(IR_QUERY_OUTPUT, "trec-eval-leaderboard")
        self.assertEqual(expected, actual)

    def test_invalid_validator_run_output(self):
        expected = [
            _ERROR,
            "No unique file in trec-eval -q format was found, only the files ['run.txt'] were available.",
        ]
        actual = check_format(VALID_RUN_OUTPUT, "trec-eval-leaderboard")
        self.assertEqual(expected, actual)

    def test_valid_example_01(self):
        expected = [_OK, "Valid trec-eval-leaderboard."]
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "trec-leaderboard").write_text(EXAMPLE_01 + EXAMPLE_02)
            actual = check_format(Path(d), "trec-eval-leaderboard")
            self.assertEqual(expected, actual)

    def test_valid_example_02(self):
        expected = [_OK, "Valid trec-eval-leaderboard."]
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "trec-leaderboard").write_text(EXAMPLE_02 + EXAMPLE_01)
            actual = check_format(Path(d), "trec-eval-leaderboard")
            self.assertEqual(expected, actual)

    def test_invalid_example_single_system_01(self):
        expected = [_ERROR, "The trec-eval-leaderboard file contains only 1 systems, this is likely an error."]
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "trec-leaderboard").write_text(EXAMPLE_01)
            actual = check_format(Path(d), "trec-eval-leaderboard")
            self.assertEqual(expected[0], actual[0])
            self.assertEqual(expected[1], actual[1])

    def test_invalid_example_single_system_02(self):
        expected = [_ERROR, "The trec-eval-leaderboard file contains only 1 systems, this is likely an error."]
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "trec-leaderboard").write_text(EXAMPLE_02)
            actual = check_format(Path(d), "trec-eval-leaderboard")
            self.assertEqual(expected[0], actual[0])
            self.assertEqual(expected[1], actual[1])

    def test_invalid_example_02_missing_all(self):
        leaderboard = (
            """
run-03 relstring             	979	'----------'
run-03 recall_5              	979	0.3"""
            + EXAMPLE_02
            + EXAMPLE_01
        )

        expected = [
            _ERROR,
            "The trec-eval-leaderboard is not valid. The all entry for",
        ]
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "trec-leaderboard").write_text(leaderboard)
            actual = check_format(Path(d), "trec-eval-leaderboard")
            self.assertEqual(expected[0], actual[0])
            self.assertIn(expected[1], actual[1])

    def test_invalid_example_03_different_measures(self):
        leaderboard = (
            "run-01 new-measure             	979	'----------'\n"
            + "run-01 new-measure             	all	'----------'"
            + EXAMPLE_02
            + EXAMPLE_01
        )

        expected = [
            _ERROR,
            "The trec-eval-leaderboard is not valid. Some measures are not available",
        ]
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "trec-leaderboard").write_text(leaderboard)
            actual = check_format(Path(d), "trec-eval-leaderboard")
            self.assertEqual(expected[0], actual[0])
            self.assertIn(expected[1], actual[1])

    def test_invalid_example_03_different_query(self):
        leaderboard = "run-01 relstring             	980	'----------'" + EXAMPLE_02 + EXAMPLE_01

        expected = [
            _ERROR,
            "The trec-eval-leaderboard is not valid. Some queries are not evaluated",
        ]
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "trec-leaderboard").write_text(leaderboard)
            actual = check_format(Path(d), "trec-eval-leaderboard")
            self.assertEqual(expected[0], actual[0])
            self.assertIn(expected[1], actual[1])



import unittest

from tira.tira_redirects import STATIC_REDIRECTS, TASKS_WITH_REDIRECT_MERGING, TIRA_ZENODO_BASE_URL


class TestTiraRedirectsMerging(unittest.TestCase):
    def test_with_maually_created_entry(self):
        expected = {
            "urls": [
                "https://zenodo.org/records/10743990/files/2023-01-07-22-09-56.zip?download=1",
                f"{TIRA_ZENODO_BASE_URL}/pyterrier-indexes/2023-01-07-22-09-56.zip",
            ],
            "run_id": "2023-01-07-22-09-56",
        }
        actual = STATIC_REDIRECTS["ir-benchmarks"]["tira-ir-starter"]["Index (tira-ir-starter-pyterrier)"][
            "msmarco-passage-trec-dl-2019-judged-20230107-training"
        ]

        self.assertEqual(expected, actual)

    def test_with_automatically_added_reneuir_entry_with_class(self):
        expected = {
            "urls": [f"{TIRA_ZENODO_BASE_URL}/reneuir-2024/runs/bi-encoder-retriever.zip?download=1"],
            "run_id": "2024-07-06-05-43-12",
        }
        actual = STATIC_REDIRECTS["reneuir-2024"]["reneuir-baselines"]["plaid-x-retrieval"][
            "dl-top-1000-docs-20240701-training"
        ]

        self.assertEqual(expected, actual)

    def test_with_automatically_added_reneuir_entry_without_class_01(self):
        expected = {
            "urls": [f"{TIRA_ZENODO_BASE_URL}/reneuir-2024/runs/ows-pyterrier-anceindex.zip?download=1"],
            "run_id": "2024-07-07-14-05-38",
        }
        actual = STATIC_REDIRECTS["reneuir-2024"]["ows"]["pyterrier-anceindex"]["dl-top-10-docs-20240701-training"]

        self.assertEqual(expected, actual)

    def test_with_automatically_added_reneuir_entry_without_class_02(self):
        expected = {
            "urls": [
                f"{TIRA_ZENODO_BASE_URL}/reneuir-2024/runs/tira-ir-starter-index-tira-ir-starter-pyterrier.zip?"
                "download=1"
            ],
            "run_id": "2024-07-07-13-27-26",
        }
        actual = STATIC_REDIRECTS["reneuir-2024"]["tira-ir-starter"]["Index (tira-ir-starter-pyterrier)"][
            "ms-marco-100-queries-20240629-training"
        ]

        self.assertEqual(expected, actual)

    def test_tasks_with_redirect_merging(self):
        expected = ["ir-benchmarks", "ir-lab-sose-2024", "reneuir-2024"]

        actual = sorted(list(TASKS_WITH_REDIRECT_MERGING))

        self.assertEqual(expected, actual)



import unittest

from tira.check_format import check_format

from . import _ERROR, _OK, EMPTY_OUTPUT, MAWSA_PROBLEMS, MAWSA_SOLUTIONS, MAWSA_TRUTHS


class TestMultiAuthorStyle(unittest.TestCase):
    def test_invalid_validator_on_empty_output_for_problems(self):
        expected = "no files matching the multi-author-style file pattern of 'problem-*.txt' in the directory"
        actual = check_format(EMPTY_OUTPUT, "multi-author-writing-style-analysis-problems")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_invalid_validator_on_empty_output_for_truths(self):
        expected = "no files matching the multi-author-style file pattern of 'truth-problem-*.json' in the directory"
        actual = check_format(EMPTY_OUTPUT, "multi-author-writing-style-analysis-truths")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_invalid_validator_on_empty_output_for_solutions(self):
        expected = "no files matching the multi-author-style file pattern of 'solution-problem-*.json' in the directory"
        actual = check_format(EMPTY_OUTPUT, "multi-author-writing-style-analysis-solutions")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_invalid_output_for_problems_01(self):
        expected = "no files matching the multi-author-style file pattern of 'truth-problem-*.json' in the directory"
        actual = check_format(MAWSA_PROBLEMS, "multi-author-writing-style-analysis-truths")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_invalid_output_for_problems_02(self):
        expected = "no files matching the multi-author-style file pattern of 'solution-problem-*.json' in the directory"
        actual = check_format(MAWSA_PROBLEMS, "multi-author-writing-style-analysis-solutions")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_invalid_output_for_solutions_01(self):
        expected = "no files matching the multi-author-style file pattern"
        actual = check_format(MAWSA_SOLUTIONS, "multi-author-writing-style-analysis-problems")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_invalid_output_for_solutions_02(self):
        expected = "no files matching the multi-author-style file pattern"
        actual = check_format(MAWSA_SOLUTIONS, "multi-author-writing-style-analysis-truths")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_invalid_output_for_truths_01(self):
        expected = "no files matching the multi-author-style file pattern"
        actual = check_format(MAWSA_TRUTHS, "multi-author-writing-style-analysis-problems")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_invalid_output_for_truths_02(self):
        expected = "no files matching the multi-author-style file pattern"
        actual = check_format(MAWSA_TRUTHS, "multi-author-writing-style-analysis-solutions")
        self.assertIn(expected, actual[1])
        self.assertEqual(_ERROR, actual[0])

    def test_valid_output_for_problems(self):
        expected = "Valid Multi-Author-Writing-Style directory"
        actual = check_format(MAWSA_PROBLEMS, "multi-author-writing-style-analysis-problems")
        self.assertIn(expected, actual[1])
        self.assertEqual(_OK, actual[0])

    def test_valid_output_for_truths(self):
        expected = "Valid Multi-Author-Writing-Style directory with"
        actual = check_format(MAWSA_TRUTHS, "multi-author-writing-style-analysis-truths")
        self.assertIn(expected, actual[1])
        self.assertEqual(_OK, actual[0])

    def test_valid_output_for_solutions(self):
        expected = "Valid Multi-Author-Writing-Style directory"
        actual = check_format(MAWSA_SOLUTIONS, "multi-author-writing-style-analysis-solutions")
        self.assertIn(expected, actual[1])
        self.assertEqual(_OK, actual[0])



import unittest

from approvaltests import verify_as_json

from tests.test_utils import digest_of_dataset, digest_of_run_output


class ReNeuIr2024Test(unittest.TestCase):
    """Unit tests for all use cases of the reneuir shared task at SIGIR 2024."""

    def test_dl_10_corpus_is_available(self):
        actual = digest_of_dataset("dl-top-10-docs-20240701-training")
        verify_as_json(actual)

    def test_dl_100_corpus_is_available(self):
        actual = digest_of_dataset("dl-top-100-docs-20240701-training")
        verify_as_json(actual)

    def test_dl_1000_corpus_is_available(self):
        actual = digest_of_dataset("dl-top-1000-docs-20240701-training")
        verify_as_json(actual)

    def test_dev_queries_100_corpus_is_available(self):
        actual = digest_of_dataset("ms-marco-100-queries-20240629-training")
        verify_as_json(actual)

    def test_dev_queries_1000_corpus_is_available(self):
        actual = digest_of_dataset("ms-marco-1000-queries-20240629-training")
        verify_as_json(actual)

    def test_dev_queries_all_corpus_is_available(self):
        actual = digest_of_dataset("ms-marco-all-dev-queries-20240629-training")
        verify_as_json(actual)

    def test_rerank_spot_check_is_available(self):
        actual = digest_of_dataset("re-rank-spot-check-20240624-training")
        verify_as_json(actual)

    def test_tiny_sample_is_available(self):
        actual = digest_of_dataset("tiny-sample-20231030-training")
        verify_as_json(actual)

    def test_dl_10_truth_corpus_is_available(self):
        actual = digest_of_dataset("dl-top-10-docs-20240701-training", truth=True)
        verify_as_json(actual)

    def test_dl_100_truth_corpus_is_available(self):
        actual = digest_of_dataset("dl-top-100-docs-20240701-training", truth=True)
        verify_as_json(actual)

    def test_dl_1000_truth_corpus_is_available(self):
        actual = digest_of_dataset("dl-top-1000-docs-20240701-training", truth=True)
        verify_as_json(actual)

    def test_dev_queries_100_truth_corpus_is_available(self):
        actual = digest_of_dataset("ms-marco-100-queries-20240629-training", truth=True)
        verify_as_json(actual)

    def test_dev_queries_1000_truth_corpus_is_available(self):
        actual = digest_of_dataset("ms-marco-1000-queries-20240629-training", truth=True)
        verify_as_json(actual)

    def test_dev_queries_all_truth_corpus_is_available(self):
        actual = digest_of_dataset("ms-marco-all-dev-queries-20240629-training", truth=True)
        verify_as_json(actual)

    def test_run_digest_of_bi_encoder(self):
        approaches = ["reneuir-2024/reneuir-baselines/plaid-x-retrieval", "reneuir-2024/ows/pyterrier-anceretrieval"]
        dataset_id = "dl-top-1000-docs-20240701-training"

        run_ids = {
            "reneuir-2024": {
                "reneuir-baselines": {"plaid-x-retrieval": {dataset_id: "2024-07-06-05-43-12"}},
                "ows": {"pyterrier-anceretrieval": {dataset_id: "2024-07-07-15-29-55"}},
            }
        }

        actual = {}
        for approach in approaches:
            actual[approach] = digest_of_run_output(approach, dataset_id, run_ids)

        verify_as_json(actual)

    def test_run_digest_of_cross_encoder(self):
        approaches = [
            "reneuir-2024/reneuir-baselines/llm-rankers-flan-t5-xl-top-100",
            "reneuir-2024/tira-ir-starter/MonoBERT Small (tira-ir-starter-gygaggle)",
        ]
        dataset_id = "dl-top-1000-docs-20240701-training"

        run_ids = {
            "reneuir-2024": {
                "reneuir-baselines": {"llm-rankers-flan-t5-xl-top-100": {dataset_id: "2024-07-04-05-31-10"}},
                "tira-ir-starter": {"MonoBERT Small (tira-ir-starter-gygaggle)": {dataset_id: "2024-07-07-12-53-42"}},
            }
        }

        actual = {}
        for approach in approaches:
            actual[approach] = digest_of_run_output(approach, dataset_id, run_ids)

        verify_as_json(actual)

    def test_run_digest_of_sparse_retriever(self):
        approaches = ["reneuir-2024/reneuir-baselines/pyterrier-bm25", "reneuir-2024/reneuir-baselines/anserini-bm25"]
        dataset_id = "dl-top-1000-docs-20240701-training"

        run_ids = {
            "reneuir-2024": {
                "reneuir-baselines": {
                    "pyterrier-bm25": {dataset_id: "2024-07-07-15-39-10"},
                    "anserini-bm25": {dataset_id: "2024-07-07-08-37-43"},
                },
            }
        }

        actual = {}
        for approach in approaches:
            actual[approach] = digest_of_run_output(approach, dataset_id, run_ids)

        verify_as_json(actual)

    def test_run_digest_of_indexes(self):
        approaches = [
            "reneuir-2024/reneuir-baselines/plaid-x-index",
            "reneuir-2024/reneuir-baselines/anserini-index",
            "reneuir-2024/ows/pyterrier-anceindex",
            "reneuir-2024/tira-ir-starter/Index (tira-ir-starter-pyterrier)",
        ]
        dataset_id = "dl-top-1000-docs-20240701-training"

        run_ids = {
            "reneuir-2024": {
                "reneuir-baselines": {
                    "plaid-x-index": {dataset_id: "2024-07-05-14-44-15"},
                    "anserini-index": {dataset_id: "2024-07-07-08-09-13"},
                },
                "ows": {
                    "pyterrier-anceindex": {dataset_id: "2024-07-07-13-43-04"},
                },
                "tira-ir-starter": {
                    "Index (tira-ir-starter-pyterrier)": {dataset_id: "2024-07-07-12-47-53"},
                },
            }
        }

        actual = {}
        for approach in approaches:
            actual[approach] = digest_of_run_output(approach, dataset_id, run_ids)

        verify_as_json(actual)



from pathlib import Path

from tira.check_format import FormatMsgType

_ERROR = FormatMsgType.ERROR
_OK = FormatMsgType.OK
_WARN = FormatMsgType.WARN

RESOURCES = Path(__file__).parent.parent / "resources"
VALID_RUN_OUTPUT = RESOURCES / "ranking-outputs"
VALID_QREL_PATH = RESOURCES / "valid-qrels"
VALID_RUN_WITH_METADATA_OUTPUT = RESOURCES / "ranking-with-metadata"
RUN_OUTPUT_WITH_TOO_FEW_QUERIES = RESOURCES / "ranking-output-invalid-too-few-queries"
RUN_OUTPUT_WITH_DUPLICATE_DOCUMENTS = RESOURCES / "ranking-output-invalid-duplicate-documents"
RUN_OUTPUT_WITH_TOO_FEW_COLUMNS = RESOURCES / "ranking-output-invalid-too-few-columns"
EMPTY_OUTPUT = RESOURCES / "input-run-01" / "1"
LSR_BENCHMARK_INPUTS = RESOURCES / "example-datasets" / "learned-sparse-retrieval"
IDEOLOGY_AND_POWER_PREDICTIONS = RESOURCES / "ideology-and-power-predictions"
IDEOLOGY_AND_POWER_LABELS = RESOURCES / "ideology-and-power-labels"
MAWSA_PROBLEMS = RESOURCES / "multi-author-writing-style-analysis-problems"
MAWSA_SOLUTIONS = RESOURCES / "multi-author-writing-style-analysis-solutions"
MAWSA_TRUTHS = RESOURCES / "multi-author-writing-style-analysis-truths"
IR_DOCUMENT_OUTPUT = RESOURCES / "valid-document-processor"
IR_DATASET = RESOURCES / "valid-irds-corpus"
LIGTHNING_IR_DOCUMENT_EMBEDDINGS = RESOURCES / "lightning-ir" / "document-embeddings"
LIGTHNING_IR_QUERY_EMBEDDINGS = RESOURCES / "lightning-ir" / "query-embeddings"
TEXT_ALIGNMENT_CORPUS_VALID_1 = RESOURCES / "text-alignment" / "1"
TEXT_ALIGNMENT_CORPUS_VALID_2 = RESOURCES / "text-alignment" / "2"
TEXT_ALIGNMENT_CORPUS_INVALID = RESOURCES / "text-alignment" / "invalid"
TEXT_ALIGNMENT_FEATURES_VALID = RESOURCES / "text-alignment" / "features"
TSV_OUTPUT_WITH_VARYING_COLUMNS = RESOURCES / "tsv-varying-columns"
TSV_OUTPUT_VALID = RESOURCES / "tsv-valid"
AGGREGATED_RESULTS_OUTPUT_VALID = RESOURCES / "aggregated-results-correct"
IR_METADATA_SINGLE_VALID = RESOURCES / "valid-single-ir-metadata"
IR_METADATA_INVALID = RESOURCES / "invalid-ir-metadata"
IR_METADATA_MULTIPLE_VALID = RESOURCES / "valid-multiple-ir-metadata"
IR_QUERY_OUTPUT = RESOURCES / "query-processing-outputs" / "query-segmentation"
JSONL_OUTPUT_VALID = RESOURCES / "jsonl-valid"
WOWS26_INCOMPLETE = RESOURCES / "wows26" / "incomplete"
WOWS26_COMPLETE = RESOURCES / "wows26" / "complete"
GEN_IR_SIM_OUTPUT_VALID = RESOURCES / "gen-ir-sim-valid"
JSONL_GZ_OUTPUT_VALID = RESOURCES / "jsonl-valid-gz"
TOUCHE_IMAGE_RETRIEVAL = RESOURCES / "touche-image-retrieval-valid"
JSONL_OUTPUT_INVALID = RESOURCES / "jsonl-invalid"
STYLE_CHANGE_CORPUS_VALID = RESOURCES / "pan24-style-change-detection"
STYLE_CHANGE_PREDICTIONS_VALID = RESOURCES / "pan24-style-change-detection-prediction"



from pathlib import Path


def tirex_tracker_mounts_or_none():
    tracker = find_tirex_tracker_executable_or_none()
    if tracker:
        return {str(tracker): {"bind": "/tracked", "mode": "ro"}}


def find_tirex_tracker_executable_or_none():
    # wget https://github.com/tira-io/tirex-tracker/releases/download/0.2.5/measure-0.2.5-linux -O 'tirex-tracker-0.2.5-linux'
    # chmod +x tirex-tracker-0.2.5-linux
    ret = Path(__file__).parent / "tirex-tracker-0.2.5-linux"
    ret = ret.resolve().absolute()

    return ret if ret.exists() else None



import json
import os
import shutil
import tempfile
import uuid
import zipfile
from abc import ABC
from pathlib import Path
from typing import TYPE_CHECKING, Callable, Dict, List, Optional, overload

from tira.check_format import _fmt, check_format, lines_if_valid, log_message

if TYPE_CHECKING:
    import io
    from typing import Any, List, Optional, Union

    import pandas as pd

# .. todo:: this file needs further documentation


class TiraClient(ABC):
    def all_datasets(self) -> "pd.DataFrame":
        pass

    def all_softwares(self) -> "pd.DataFrame":
        pass

    def print_overview_of_all_software(self) -> None:
        pass

    def all_evaluators(self) -> "pd.DataFrame":
        pass

    def all_evaluated_appraoches(self) -> "pd.DataFrame":
        pass

    def docker_software(self) -> "Any":
        # .. todo:: typehint
        pass

    def run_was_already_executed_on_dataset(self, approach: str, dataset: str) -> bool:
        raise ValueError("ToDo: Implement")

    def get_run_output(self, approach: str, dataset: str, allow_without_evaluation: bool = False) -> Path:
        raise ValueError("ToDo: Implement")

    def claim_ownership(self, uuid: str, team: str, system: str, description: str, task_id: str) -> Dict:
        raise ValueError("ToDo: Implement")

    def get_dataset(self, dataset: str) -> dict:
        """Get the TIRA representation of an dataset identified by the passed dataset argument.

        Args:
            dataset (Union[str, IrDataset, PyTerrierDataset): The dataset that is either the string id of the dataset in TIRA, the string id of an ir_dataset, the string id of an PyTerrier dataset, or an instantiation of an ir_dataset or an PyTerrier dataset.
        Returns:
            dict: The TIRA representation of the dataset.
        """
        raise ValueError("ToDo: Implement")

    def download_dataset(
        self, task: str, dataset: str, truth_dataset: bool = False, allow_local_dataset: bool = False
    ) -> Path:
        """
        Download the dataset. Set truth_dataset to true to load the truth used for evaluations.
        """
        raise ValueError("ToDo: Implement")

    def json_response(
        self,
        endpoint: str,
        params: "Optional[Union[Dict, List[tuple], bytes]]" = None,
        base_url: "Optional[str]" = None,
        failsave_retries: "Optional[int]" = None,
    ) -> Dict:
        raise ValueError("ToDo: Implement")

    def api_key_is_valid(self) -> bool:
        try:
            role = self.json_response("/api/role")
        except:
            return False

        if (
            (self.api_user_name is None or self.api_user_name == "no-api-key-user")
            and role
            and "context" in role
            and "user_id" in role["context"]
            and role["context"]["user_id"]
        ):
            self.api_user_name = role["context"]["user_id"]

        return (
            role
            and "status" in role
            and "role" in role
            and "csrf" in role
            and role["csrf"]
            and 0 == role["status"]
            and "user_id" in role["context"]
            and role["context"]["user_id"]
            and "role" in role["context"]
            and role["context"]["role"]
            and "guest" != role["context"]["role"]
        )

    def fail_if_api_key_is_invalid(self) -> None:
        if not self.api_key_is_valid():
            role = self.json_response("/api/role")
            raise ValueError("It seems like the api key is invalid. Got: ", role)

    def get_run_execution_or_none(self, approach, dataset, previous_stage_run_id=None) -> "Optional[dict[str, str]]":
        # .. todo:: typehint
        pass

    def docker_registry(self):
        return "registry.webis.de"

    def modify_task(self, task_id: str, to_rename: "dict[str, Any]"):
        # .. todo:: typehint
        pass

    @overload
    def download_run(
        self,
        task,
        dataset,
        software,
        team=None,
        previous_stage=None,
        return_metadata: bool = False,
    ) -> "tuple[pd.DataFrame, str]":
        # .. todo:: typehint
        ...

    @overload
    def download_run(
        self,
        task,
        dataset,
        software,
        team=None,
        previous_stage=None,
        return_metadata: bool = False,
    ) -> "pd.DataFrame":
        # .. todo:: typehint
        ...

    def download_run(
        self,
        task,
        dataset,
        software,
        team=None,
        previous_stage=None,
        return_metadata: bool = False,
    ) -> "Union[pd.DataFrame, tuple[pd.DataFrame, str]]":
        # .. todo:: typehint
        pass

    def create_new_upload(self, task_id: str, vm_id: str) -> "Optional[str]":
        """
        Creates a new upload and returns the newly created id. Returns None on failure.
        """
        pass

    def submit_run(self, task_id: str, vm_id: str, dataset_id: str, upload_id: str, run: "io.IOBase") -> bool:
        """
        Submits the runfile located at `run` for the given task and vm for the given upload id. Returns true on success,
        false  otherwise.
        """
        pass

    def _git_repo(self, path: Path):
        import git

        try:
            # TODO: Replace with call to tirex_tracker.
            return git.Repo(path, search_parent_directories=True)
        except git.exc.InvalidGitRepositoryError as e:
            raise ValueError(f"No git repository found at {path}") from e

    def _zip_tracked_files(self, repo: "git.Repo", directory: str):
        """
        Creates a zip archive containing all tracked files in a given Git repository.

        :param repo: The Git repository.
        :param zip_filename: Name of the output zip file.
        """
        # TODO: Replace with call to tirex_tracker.
        tracked_files = [i.path for i in repo.commit().tree.traverse() if i.path.startswith(f"{directory}/")]
        zip_path = Path(tempfile.TemporaryDirectory().name) / "repo.zip"
        zip_path.parent.mkdir(exist_ok=True, parents=True)

        with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as zipf:
            for file in tracked_files:
                file_path = os.path.join(repo.working_tree_dir, file)
                zipf.write(file_path, arcname=file)
        return zip_path

    def __run_evaluation(
        self,
        image: str,
        command: str,
        predictions: Path,
        truths: Path,
        print_message: Callable,
        ret_dir: Optional[Path] = None,
    ) -> str:
        from tira.io_utils import load_output_of_directory
        from tira.third_party_integrations import temporary_directory

        if not self.local_execution.docker_is_installed_failsave():
            msg = "Docker is not installed, I can not run the dockerized evaluator..."
            log_message(msg, _fmt.ERROR)
            raise ValueError(msg)

        print_message("Docker is installed. I can run the dockerized evaluator.", _fmt.OK)
        self.local_execution.ensure_image_available_locally(image)

        print_message(f"The evaluator image {image} is available locally.", _fmt.OK)

        if ret_dir is None:
            ret_dir = temporary_directory()

        eval_config = {
            "evaluator_id": "1",
            "evaluator_git_runner_image": image,
            "evaluator_git_runner_command": command,
            "truth_directory": str(Path(truths).absolute().resolve()),
        }
        self.local_execution.evaluate(
            Path(ret_dir).absolute().resolve(),
            predictions.absolute().resolve(),
            allow_network=False,
            evaluate=eval_config,
        )

        try:
            return json.dumps(load_output_of_directory(Path(ret_dir), evaluation=True))
        except:
            msg = "The evaluator failed to produce a valid evaluation..."
            print(msg)
            raise ValueError(msg)

    def evaluate(self, directory: Path, dataset_id: str, results_dir: Optional[Path] = None) -> None:
        """Evaluate some predictions made for some dataset on your local machine.

        Args:
            directory (Path): The path to the directory that contains the predictions that you want to evaluate.
            dataset_id (str): The ID of the TIRA dataset on which the directory is to be evaluated.
        """
        all_messages = []

        def print_message(message, level):
            all_messages.append((message, level))
            os.system("cls" if os.name == "nt" else "clear")
            print(f"TIRA Evaluation on '{dataset_id}':")
            for m, l in all_messages:
                log_message(m, l)

        dataset_handle = self.get_dataset(dataset_id)
        if (
            not dataset_handle
            or "mirrors" not in dataset_handle
            or "inputs" not in dataset_handle["mirrors"]
            or "format" not in dataset_handle
            or len(dataset_handle["format"]) == 0
        ):
            raise ValueError("Dataset configuration is invalid: " + str(dataset_handle))

        if (
            "evaluator" not in dataset_handle
            or "image" not in dataset_handle["evaluator"]
            or "command" not in dataset_handle["evaluator"]
        ):
            raise ValueError("Dataset configuration has no evaluator configured: " + str(dataset_handle))

        if not directory or not directory.exists():
            print_message(f"The directory {directory} does not exist.", _fmt.ERROR)
            raise ValueError(f"The directory {directory} does not exist.")

        result, msg = check_format(directory, dataset_handle["format"][0])
        if result != _fmt.OK:
            log_message(msg, result)
            raise ValueError(msg)
        print_message(f"The predictions in {directory} have the expected format.", _fmt.OK)

        dataset_path = self.download_dataset(dataset_handle["default_task"], dataset_id, truth_dataset=True)
        print_message(f"The dataset {dataset_id} is available locally.", _fmt.OK)

        preds = self.__run_evaluation(
            dataset_handle["evaluator"]["image"],
            dataset_handle["evaluator"]["command"],
            directory,
            dataset_path,
            print_message,
            results_dir,
        )

        log_message("The evaluator was successfull.", _fmt.OK)
        print("\n\nResult:\n\t" + preds)

    def __extract_task_and_dataset_id(self, task, dataset):
        if dataset is None and task and len(task.split("/")) == 2:
            task, dataset = task.split("/")
        elif dataset is None:
            task, dataset = None, task

        return task, dataset

    def iter_run_output(self, approach, dataset, format):
        from tira.ir_datasets_util import translate_irds_id_to_tirex

        if dataset is None and os.path.exists(approach):
            output_dir = approach
        else:
            output_dir = self.get_run_output(approach, translate_irds_id_to_tirex(dataset))
        for i in lines_if_valid(Path(output_dir), format):
            yield i

    def iter_dataset(self, task, dataset, truth_dataset, format):
        local_dir = task
        task, dataset = self.__extract_task_and_dataset_id(task, dataset)

        if not self.dataset_exists_in_tira(dataset) and Path(local_dir).exists():
            dataset_dir = local_dir
        else:
            dataset_dir = self.download_dataset(task, dataset, truth_dataset)

        for i in lines_if_valid(Path(dataset_dir), format):
            yield i

    def clone_git_repository(self, repo_url: str):
        from git import Repo

        target_file = Path(self.tira_cache_dir) / "git-repositories" / (repo_url.split("/")[-1])
        if target_file.exists():
            return target_file

        target_file.parent.mkdir(exist_ok=True, parents=True)
        Repo.clone_from(repo_url, target_file)
        return target_file

    def build_docker_image_from_code(
        self, path: Path, print_message, dry_run: bool, docker_file: "Optional[Path]" = None
    ):
        repo = self._git_repo(path)

        try:
            remotes = {remote.name: remote.url for remote in repo.remotes}
        except:
            raise ValueError("No remotes found for the git repository.")

        if len(remotes) == 0:
            raise ValueError("No remotes found for the git repository.")

        try:
            commit = repo.commit().hexsha
        except:
            raise ValueError("No commits in the git repository")

        try:
            active_branch = repo.active_branch.name
        except:
            raise ValueError("No branch in the git repository")

        print_message(f"The code is in a git repository {repo.working_tree_dir}.", _fmt.OK)

        if docker_file is None:
            docker_file = path / "Dockerfile"

        docker_file = Path(docker_file)
        if not docker_file.exists():
            raise ValueError(f"No dockerfile {docker_file} exists.")

        directory_in_path = str(Path(path).absolute()).replace(str(Path(repo.working_tree_dir).absolute()) + "/", "")
        if Path(repo.working_tree_dir).name in directory_in_path:
            directory_in_path = Path(repo.working_tree_dir).name
        submission_name = (
            directory_in_path.replace("/", "-").lower().replace(" ", "-").replace("\n", "").replace("\r", "")
        )
        docker_tag = submission_name + "-" + str(uuid.uuid4())[:5]

        if repo.is_dirty(untracked_files=True):
            if not dry_run:
                log_message(
                    f"The git repository {repo.working_tree_dir} is not clean.\n\tPlease ensure that the repository is clean, i.g., git status reports that everything is committed and pushed.\n\n\tPlease pass --dry-run if you want to test without uploading",
                    _fmt.ERROR,
                )
                raise ValueError("The git repository is not clean.")
        else:
            print_message(f"The git repository {repo.working_tree_dir} is clean.", _fmt.OK)
        print("Build Docker image...")
        zipped_code = self._zip_tracked_files(repo, directory_in_path)

        self.local_execution.build_docker_image(path, docker_tag, docker_file)

        print_message(f"The code is embedded into the docker image {docker_tag}.", _fmt.OK)
        return docker_tag, zipped_code, remotes, commit, active_branch

    def submit_code(
        self,
        path: Path,
        task_id: str,
        command: "Optional[str]" = None,
        dataset_id: "Optional[str]" = None,
        user_id: "Optional[str]" = None,
        docker_file: "Optional[Path]" = None,
        dry_run: "Optional[bool]" = False,
        allow_network: "Optional[bool]" = False,
        mount_hf_model: "Optional[list[str]]" = None,
    ):
        """Build a tira submission from a git repository.

        Args:
            path (Path): The path to the directory that contains the submission. The path points to the directory used as root for the docker build and must be in a git repository.
            task_id (str): The ID of the TIRA task to which the submissions should be made.
            dataset_id (str, optional): The ID of the TIRA dataset on which the submission is to be tested. If no dataset is passed, the submission will be tested on a small publicly available smoke test dataset for the task.
            user_id (str, optional): The ID of the TIRA team that makes the submission. Is only required if a user has multiple teams.
            docker_file (Path, optional): The Dockerfile to build the submission within the repository. Defaults to None to use path/Dockerfile.
        """
        from tira.third_party_integrations import temporary_directory

        all_messages = []

        def print_message(message, level):
            all_messages.append((message, level))
            os.system("cls" if os.name == "nt" else "clear")
            print("TIRA Code Submission:")
            for m, l in all_messages:
                log_message(m, l)

        if not dry_run:
            print(f"Ensure that you are registed for task {task_id}.")
            from tira.tira_run import guess_vm_id_of_user

            user_id = guess_vm_id_of_user(task_id, self, user_id)
            assert user_id
            print_message(f"You are registered and will submit as team {user_id} to task {task_id}.", _fmt.OK)

        if dataset_id is None:
            for k, v in self.datasets(task_id).items():
                if (
                    "is_confidential" not in v
                    or v["is_confidential"]
                    or "is_deprecated" not in v
                    or v["is_deprecated"]
                    or "format" not in v
                    or len(v["format"]) == 0
                ):
                    continue
                candidate = self.get_dataset(f"{task_id}/{k}")
                if not candidate or "mirrors" not in candidate or "inputs" not in candidate["mirrors"]:
                    continue
                dataset_id = k

            if dataset_id is None:
                raise ValueError("foo")

        dataset_handle = self.get_dataset(f"{task_id}/{dataset_id}")
        if (
            not dataset_handle
            or "mirrors" not in dataset_handle
            or "inputs" not in dataset_handle["mirrors"]
            or "format" not in dataset_handle
            or len(dataset_handle["format"]) == 0
        ):
            raise ValueError("foo")

        dataset_path = self.download_dataset(task_id, dataset_id)
        print_message(f"The dataset {dataset_id} is available locally.", _fmt.OK)

        docker_tag, zipped_code, remotes, commit, active_branch = self.build_docker_image_from_code(
            path, print_message, dry_run, docker_file
        )
        print("Test Docker image...")

        hf_models = None
        if mount_hf_model:
            from tira.io_utils import huggingface_model_mounts

            hf_models = huggingface_model_mounts(mount_hf_model)
            hf_models = [k + ":" + v["bind"] + ":" + v["mode"] for k, v in hf_models.items()]
            print(f"The following models from huggingface are mounted: {hf_models}\n\n")

        if command is None:
            command = self.local_execution.extract_entrypoint(docker_tag)

        tmp_dir = temporary_directory()
        self.local_execution.run(
            image=docker_tag,
            command=command,
            input_dir=dataset_path,
            output_dir=tmp_dir,
            allow_network=allow_network,
            additional_volumes=hf_models,
        )

        format_configuration = (
            None if "format_configuration" not in dataset_handle else dataset_handle["format_configuration"]
        )
        result, msg = check_format(Path(tmp_dir), dataset_handle["format"], format_configuration)
        if result != _fmt.OK:
            print(msg)
            raise ValueError(msg)
        print_message(f"The docker image produced valid outputs on the dataset {dataset_id}.", _fmt.OK)
        shutil.copy(zipped_code, Path(tmp_dir) / "source-code.zip")

        if not dry_run:
            print("Upload Code Submission image...")
            metadata_uuid = self.upload_run_anonymous(tmp_dir, dataset_id)["uuid"]

            print_message("The meta data is uploaded to TIRA.", _fmt.OK)

            print("Push Docker image to TIRA...")
            from tira.tira_run import push_image

            pushed_image = push_image(self, docker_tag, task_id, user_id)

            print_message("The Docker image is pushed to TIRA.", _fmt.OK)
            print("Configure code submission in TIRA...")
            upload = self.add_docker_software(
                pushed_image,
                command,
                user_id,
                task_id,
                None,
                dict(os.environ),
                source_code_remotes=remotes,
                source_code_commit=commit,
                source_code_active_branch=active_branch,
                try_run_metadata_uuid=metadata_uuid,
                mount_hf_model=mount_hf_model,
            )
            print_message(f"The code submission is uploaded to TIRA.", _fmt.OK)
            print("\nResult:")
            log_message(
                f"Your code submission is available in TIRA as {upload['display_name']}.",
                _fmt.OK,
            )

        return {
            "code": zipped_code,
            "remotes": remotes,
            "commit": commit,
            "active_branch": active_branch,
            "image": docker_tag,
        }

    def submit_dataset(
        self,
        path: Path,
        task: str,
        split: str,
        dry_run: bool,
        system_inputs: str = "inputs",
        truths: str = "truths",
        skip_baseline: bool = False,
    ):
        from shutil import copy

        from tira.io_utils import TqdmUploadFile, verify_tira_installation, zip_dir
        from tira.third_party_integrations import temporary_directory

        all_messages = []

        def print_message(message: str, level: _fmt) -> None:
            all_messages.append((message, level))
            os.system("cls" if os.name == "nt" else "clear")
            print("TIRA Dataset Submission:")
            for m, l in all_messages:
                log_message(m, l)

        if not dry_run:
            result = verify_tira_installation()
            if result != _fmt.OK:
                msg = "Your tira installation is not valid. Please run: tira-cli verify-installation"
                print_message(msg, _fmt.ERROR)
                return None

        if path is None or not path.exists() or not path.is_dir():
            msg = f"The path {path} does not exist. I can not continue"
            log_message(msg, _fmt.ERROR)
            return None

        print_message("Your tira installation is valid.", _fmt.OK)

        try:
            from datasets import load_dataset, load_dataset_builder
        except ImportError as e:
            msg = "I could not import the datasets library to load the dataset. Please install the datasets library (e.g., via pip3 install datasets)."
            log_message(msg, _fmt.ERROR)
            return None

        try:
            from huggingface_hub import DatasetCard
        except ImportError as e:
            msg = "I could not import the huggingface_hub library to load the DatasetCard. Please install the huggingface_hub library (e.g., via pip3 install huggingface_hub)."
            log_message(msg, _fmt.ERROR)
            return None

        if str(path).strip() in (".", ".."):
            print_message(
                f"Please do not pass '.' or '..' as argument for --path as the datasets library uses the name of the directory.",
                _fmt.ERROR,
            )
            return None

        if not (path / "README.md").is_file():
            print_message(
                f"I expect a huggingface dataset configuration in a file {path / 'README.md'}.",
                _fmt.ERROR,
            )
            return None

        tira_configs = DatasetCard.load(str(path / "README.md")).data["tira_configs"]
        resolve_inputs_to = tira_configs.get("resolve_inputs_to", None)
        input_format = tira_configs["input_format"]["name"]
        input_config = tira_configs["input_format"].get("config", {})
        git_url, subdir = tira_configs["baseline"]["link"].replace("/tree/master/", "/tree/main/").split("/tree/main/")
        baseline_command = tira_configs["baseline"]["command"]
        baseline_format = tira_configs["baseline"]["format"]["name"]
        baseline_config = tira_configs["baseline"]["format"].get("config", {})
        if "measures" in tira_configs["evaluator"]:
            eval_image, eval_command, eval_measures = None, None, tira_configs["evaluator"]["measures"]
        else:
            eval_image = tira_configs["evaluator"]["image"]
            eval_command = tira_configs["evaluator"]["command"]
            eval_measures = None
        resolve_truths_to = tira_configs.get("resolve_truths_to", None)

        print_message(f"The configuration of the dataset {path} is valid.", _fmt.OK)

        truth_format = tira_configs["truth_format"]["name"]
        truth_config = tira_configs["truth_format"].get("config", {})

        baseline_format = baseline_format if isinstance(baseline_format, List) else [baseline_format]
        truth_format = truth_format if isinstance(truth_format, List) else [truth_format]

        dataset_system_inputs = load_dataset_builder(str(path), system_inputs)
        dataset_truths = load_dataset_builder(str(path), truths)

        inputs_dir = temporary_directory()
        truth_dir = temporary_directory()

        if split not in dataset_system_inputs.config.data_files:
            msg = (
                f"Split {split} is not available. Possible are: {list(dataset_system_inputs.config.data_files.keys())}"
            )
            log_message(msg, _fmt.ERROR)
            return None

        for data_file in dataset_system_inputs.config.data_files[split]:
            relative_data_file = Path(data_file).relative_to(
                Path(dataset_system_inputs.base_path).absolute() / resolve_inputs_to
            )
            target_file = inputs_dir / relative_data_file
            target_file.parent.mkdir(exist_ok=True, parents=True)
            copy(data_file, target_file)

        for data_file in dataset_truths.config.data_files[split]:
            relative_data_file = Path(data_file).relative_to(
                Path(dataset_truths.base_path).absolute() / resolve_truths_to
            )
            target_file = truth_dir / relative_data_file
            target_file.parent.mkdir(exist_ok=True, parents=True)
            copy(data_file, target_file)

        l, m = check_format(inputs_dir, input_format, input_config)
        if l != _fmt.OK:
            log_message(m, l)
            return None

        print_message("The system inputs are valid.", _fmt.OK)

        l, m = check_format(truth_dir, truth_format, truth_config)
        if l != _fmt.OK:
            log_message(m, l)
            return None

        print_message("The truth data is valid.", _fmt.OK)

        if not skip_baseline:
            git_repo_local = self.clone_git_repository(git_url)
            print_message(f"Repository for the baseline is cloned from {git_url}.", _fmt.OK)

            docker_tag, _, _, _, _ = self.build_docker_image_from_code(
                git_repo_local / Path(subdir), log_message, False
            )

            print_message(f"The baseline {subdir} is embedded in a Docker image.", _fmt.OK)

            if baseline_command is None:
                baseline_command = self.local_execution.extract_entrypoint(docker_tag)

            baseline_output = temporary_directory()
            self.local_execution.run(
                image=docker_tag, command=baseline_command, input_dir=inputs_dir, output_dir=baseline_output
            )

            l, m = check_format(baseline_output, baseline_format, baseline_config)
            if l != _fmt.OK:
                log_message(f"The outputs of the baseline are at {baseline_output} and not valid: {m}", l)
                return None

            if eval_image is not None and eval_command is not None:
                print_message(f"The baseline produced valid outputs at {baseline_output}.", _fmt.OK)
                preds = self.__run_evaluation(eval_image, eval_command, baseline_output, truth_dir, log_message)

                print_message(f"The evaluation of the baseline produced valid outputs: {preds}.", _fmt.OK)
            else:
                from tira.evaluators import evaluate

                eval_config: Dict = {
                    "measures": eval_measures,
                    "run_format": baseline_format,
                    "truth_format": truth_format,
                }
                preds = evaluate(
                    baseline_output,
                    truth_dir,
                    eval_config,
                )
                print_message(f"The evaluation of the baseline produced valid outputs: {preds}.", _fmt.OK)

        ret = {}

        ret["inputs_dir"] = inputs_dir
        ret["truths_dir"] = truth_dir
        ret["inputs_zip"] = zip_dir(inputs_dir)
        ret["truths_zip"] = zip_dir(truth_dir)

        if not dry_run:
            headers = self.authentication_headers()
            headers["Accept"] = "application/json"
            headers["Content-Type"] = "application/json"
            self.fail_if_api_key_is_invalid()

            csrf_token = self.get_csrf_token()
            headers["Cookie"] = f"csrftoken={csrf_token}" + (
                "" if "Cookie" not in headers else "; " + headers["Cookie"]
            )
            headers["x-csrftoken"] = csrf_token
            dataset_name = path.name
            dataset_id = dataset_name.lower().replace("/", "-").replace(" ", "-").replace("_", "-")
            url = f"{self.base_url}/tira-admin/add-dataset/{task}"
            content = {
                "csrfmiddlewaretoken": csrf_token,
                "dataset_id": dataset_id,
                "name": dataset_name,
                "task": task,
                "type": "training" if ("train" in split.lower()) else "test",
                "upload_name": "predictions.jsonl",
                "is_confidential": True,
                "irds_docker_image": "",
                "irds_import_command": "",
                "irds_import_truth_command": "",
                "git_runner_image": eval_image if eval_image else "ubuntu:18.04",
                "git_runner_command": eval_command if eval_command else "echo 'this is no real evaluator'",
                "is_git_runner": True,
                "use_existing_repository": False,
                "working_directory": "obsolete",
                "command": "obsolete",
                "publish": False,
                "evaluator_command": "obsolete",
                "evaluator_image": "obsolete",
                "evaluator_working_directory": "obsolete",
                "description": "todo",
                "chatnoir_id": "",
                "ir_datasets_id": "",
                "format": baseline_format,
                "format_configuration": json.dumps(baseline_config),
                "truth_format": truth_format,
                "truth_format_configuration": json.dumps(truth_config),
            }

            if eval_measures is not None:
                content["trusted_measures"] = eval_measures

            import requests

            resp = requests.post(url, headers=headers, json=content, verify=self.verify)
            resp_content = resp.content.decode("utf8")
            try:
                dataset_id = json.loads(resp_content)["context"]["dataset_id"]
            except:
                print(resp_content)
                raise ValueError(f"Failure. Got response {resp.status_code} from server:\n\n{resp_content}")
            print_message(f"Configuration for dataset {dataset_id} is uploaded to TIRA.", _fmt.OK)

            def post_data(type: str) -> None:
                tqdm_zip_file = TqdmUploadFile(ret[f"{type}s_zip"], f"Upload {type}s to TIRA")

                headers = self.authentication_headers()
                headers["Accept"] = "application/json"
                headers["Cookie"] = f"csrftoken={csrf_token}" + (
                    "" if "Cookie" not in headers else "; " + headers["Cookie"]
                )
                headers["x-csrftoken"] = csrf_token
                files = {"file": (os.path.basename(ret[f"{type}s_zip"]), tqdm_zip_file)}

                resp = requests.post(
                    url=f"{self.base_url}/tira-admin/upload-dataset/{task}/{dataset_id}/{type}",
                    files=files,
                    headers=headers,
                    verify=self.verify,
                )

                resp_content = resp.content.decode("utf8")
                try:
                    resp_parsed = json.loads(resp_content)
                except:
                    print(resp_content)
                    raise ValueError(f"Failure. Got response {resp.status_code} from server:\n\n{resp_content}")
                if "status" not in resp_parsed or "message" not in resp_parsed or resp_parsed["status"] != 0:
                    log_message(f"Could not upload system {type}s: {resp_content}", _fmt.ERROR)
                    return

                print_message(f"{type}s are uploaded to TIRA: {resp_parsed['message']}", _fmt.OK)

            post_data("input")
            post_data("truth")

        return ret

    def __extract_dataset_identifier(self, dataset: any):
        """Extract the dataset identifier from a passed object.

        Args:
            dataset (any): Some representation of dataset.

        Returns:
            Optional[dict]: The dataset identifier if available.
        """
        if hasattr(dataset, "irds_ref"):
            return self.__extract_dataset_identifier(dataset.irds_ref())
        if hasattr(dataset, "dataset_id"):
            return dataset.dataset_id()
        return dataset

    def __matching_dataset(self, datasets, dataset_identifier) -> "Optional[dict]":
        """Find the dataset identified by the passed dataset_identifier in all passed datasets.

        Args:
            datasets (List[dict]): TIRA representations of datasets.
            dataset_identifier (_type_): identifier of the dataset to be found.

        Returns:
            Optional[dict]: The TIRA representation of the dataset if found in the datasets.
        """
        datasets = [i for i in datasets if "id" in i and len(i["id"]) > 2]

        for dataset in datasets:
            if dataset_identifier and dataset["id"] == dataset_identifier:
                return dataset

        if len(dataset_identifier.split("/")) == 2:
            task_identifier, dataset_in_task = dataset_identifier.split("/")

            for dataset in datasets:
                if "default_task" not in dataset or not dataset["default_task"]:
                    continue

                if not task_identifier or not dataset_in_task:
                    continue

                if task_identifier == dataset["default_task"] and dataset_in_task == dataset["id"]:
                    return dataset

                if task_identifier == dataset["default_task"] and dataset_in_task == dataset["display_name"]:
                    return dataset

        for dataset in datasets:
            if "ir_datasets_id" not in dataset or not dataset["ir_datasets_id"] or len(dataset["ir_datasets_id"]) < 3:
                continue

            if dataset_identifier and dataset_identifier == dataset["ir_datasets_id"]:
                return dataset

        matches = []
        for dataset in datasets:
            if "default_task" not in dataset or not dataset["default_task"]:
                continue

            if dataset_identifier and dataset_identifier == dataset["display_name"]:
                matches.append(dataset)

        if len(matches) == 1:
            return matches[0]



import unittest

from tira.check_format import check_format

from . import _ERROR, _OK, EMPTY_OUTPUT, IR_DOCUMENT_OUTPUT


class TestDocumentProcessorFormat(unittest.TestCase):
    def test_valid_document_processor_format(self):
        """Test that a valid query processor output directory passes validation."""
        expected = [_OK, "The jsonl file has the correct format."]
        actual = check_format(IR_DOCUMENT_OUTPUT, "document-processor")
        self.assertEqual(expected, actual)

    def test_error_message_on_empty_output(self):
        """Test error message when no jsonl file is found."""
        expected = [_ERROR, "No unique *.jsonl file was found, only the files ['.gitkeep'] were available."]
        actual = check_format(EMPTY_OUTPUT, "document-processor")
        self.assertEqual(expected, actual)



import unittest

from tira.check_format import check_format, lines_if_valid

from . import (
    _ERROR,
    _OK,
    EMPTY_OUTPUT,
    IR_QUERY_OUTPUT,
    TSV_OUTPUT_VALID,
    TSV_OUTPUT_WITH_VARYING_COLUMNS,
    VALID_RUN_OUTPUT,
)


class TestCheckTsvForNonExistingFormats(unittest.TestCase):
    def test_invalid_validator_on_empty_output(self):
        expected = [_ERROR, "No unique *.tsv file was found, only the files ['.gitkeep'] were available."]
        actual = check_format(EMPTY_OUTPUT, "*.tsv")
        self.assertEqual(expected, actual)

    def test_invalid_validator_on_query_output(self):
        expected = [_ERROR, "No unique *.tsv file was found, only the files ['queries.jsonl'] were available."]
        actual = check_format(IR_QUERY_OUTPUT, "*.tsv")
        self.assertEqual(expected, actual)

    def test_invalid_trec_run_file(self):
        expected = [_ERROR, "No unique *.tsv file was found, only the files ['run.txt'] were available."]
        actual = check_format(VALID_RUN_OUTPUT, "*.tsv")
        self.assertEqual(expected, actual)

    def test_invalid_tsv_with_varying_columns(self):
        expected = [_ERROR, "The *.tsv file is invalid: The number of columns varies."]
        actual = check_format(TSV_OUTPUT_WITH_VARYING_COLUMNS, "*.tsv")
        self.assertEqual(expected, actual)

    def test_valid_tsv(self):
        expected = [_OK, "The tsv file has the correct format."]
        actual = check_format(TSV_OUTPUT_VALID, "*.tsv")
        self.assertEqual(expected, actual)

    def test_lines_of_valid_tsv(self):
        expected = [["column 1", "column 2", "column 3"]] * 5
        actual = lines_if_valid(TSV_OUTPUT_VALID, "*.tsv")
        self.assertEqual(expected, actual)



import unittest

from . import load_artifact


class TestErrorMessages(unittest.TestCase):
    def test_error_message_on_empty_path(self):
        with self.assertRaises(ValueError) as cnt:
            load_artifact("tira:")
        self.assertIn("Invalid tira url. I expected", str(cnt.exception))

    def test_error_message_on_dataset_that_does_not_exist(self):
        with self.assertRaises(ValueError) as cnt:
            load_artifact("tira:i-am-not/a-data/set")
        self.assertIn("could not find a ir-dataset", str(cnt.exception))

    def test_error_message_on_missing_approach_name(self):
        with self.assertRaises(ValueError) as cnt:
            load_artifact("tira:argsme/2020-04-01/touche-2021-task-1/does-not-exist")
        self.assertIn("have no team and/or approach", str(cnt.exception))

    def test_error_message_on_missing_approach_name_02(self):
        with self.assertRaises(ValueError) as cnt:
            load_artifact("tira:argsme/2020-04-01/touche-2021-task-1/does-not-exist/a/b/c")
        self.assertIn("have no team and/or approach", str(cnt.exception))

    def test_error_message_for_non_existing_approach(self):
        with self.assertRaises(ValueError) as cnt:
            load_artifact("tira:argsme/2020-04-01/touche-2021-task-1/does-not-exist/does-not-exist")
        self.assertIn("No submission 'does-not-exist' by team 'does-not-exist' is publicly", str(cnt.exception))



import unittest

from tira.check_format import check_format

from . import _ERROR, _OK, EMPTY_OUTPUT, LSR_BENCHMARK_INPUTS


class TestQueryProcessorFormat(unittest.TestCase):
    def test_error_message_on_empty_output(self):
        actual = check_format(EMPTY_OUTPUT, "lsr-benchmark-inputs")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn("No such file or directory", actual[1])

    def test_for_valid_dataset(self):
        expected = [_OK, "The dataset is in the format for the lsr-benchmark."]
        actual = check_format(LSR_BENCHMARK_INPUTS, "lsr-benchmark-inputs")
        self.assertEqual(expected, actual)



import unittest

from tira.io_utils import extract_volume_mounts


class TestTiraIoUtils(unittest.TestCase):
    def test_simple_case_01(self):
        a, b, c = extract_volume_mounts("a:b:c")
        self.assertEqual("a", a)
        self.assertEqual("b", b)
        self.assertEqual("c", c)

    def test_simple_case_02(self):
        a, b, c = extract_volume_mounts("/workspace/tira/python-client:/foo:ro")
        self.assertEqual("/workspace/tira/python-client", a)
        self.assertEqual("/foo", b)
        self.assertEqual("ro", c)

    def test_windows_case_01(self):
        a, b, c = extract_volume_mounts("c:\\Windows:b:c")
        self.assertEqual("c:\\Windows", a)
        self.assertEqual("b", b)
        self.assertEqual("c", c)

    def test_windows_case_02(self):
        a, b, c = extract_volume_mounts("c:\\Windows:/foo:ro")
        self.assertEqual("c:\\Windows", a)
        self.assertEqual("/foo", b)
        self.assertEqual("ro", c)



import json
import os
import tempfile
from pathlib import Path
from typing import TYPE_CHECKING, Iterable, Mapping, Optional

import pandas as pd
from pyterrier.transformer import SourceTransformer, Transformer

if TYPE_CHECKING:
    import pandas as pd

    from .tira_client import TiraClient


def merge_runs(topics: "pd.DataFrame", run_file: Path) -> "pd.DataFrame":
    import numpy as np

    df = pd.read_csv(run_file, sep="\\s+", names=["qid", "q0", "docno", "rank", "score", "system"])
    df["qid"] = df["qid"].astype(str)
    df["docno"] = df["docno"].astype(str)
    topics["qid"] = topics["qid"].astype(str)

    common_columns = np.intersect1d(topics.columns, df.columns)
    join_on = ["qid", "docno"] if "qid" in common_columns and "docno" in common_columns else ["qid"]

    # we drop columns in topics that exist in the df
    keeping = topics.columns
    drop_columns = [i for i in common_columns if i not in {"qid", "docno"}]
    if len(drop_columns) > 0:
        keeping = topics.columns[~topics.columns.isin(drop_columns)]

    return topics[keeping].merge(df, how="left", left_on=join_on, right_on=join_on)


class TiraSourceTransformer(SourceTransformer):
    """Wraps a cached Tira  run as a PyTerrier transformer.

    Parameters
    ----------
    on_column_mismatch : {'warn', 'error', 'ignore'}, default='warn'
    How to react if the topics frame arriving at ``transform()`` contains
    extra query-rewrite columns (``query_0``, ``query_1``, etc.) that a *run*
    artifact cannot honour.
    """

    def __init__(self, rtr: "pd.DataFrame", *, on_column_mismatch: "Optional[str]" = None, **kwargs) -> None:
        super().__init__(rtr, **kwargs)  # type: ignore[no-untyped-call]
        if on_column_mismatch is None:
            on_column_mismatch = os.getenv("TIRA_ARTIFACT_ON_COLUMN_MISMATCH", "warn").lower()
        self.on_column_mismatch = on_column_mismatch

    def transform(self, topics):
        import numpy as np

        # fail/warn/ignore on extra columns
        extra = [c for c in topics.columns if c.startswith("query_")]
        if extra and self.on_column_mismatch != "ignore":
            import warnings

            if self.on_column_mismatch == "error":
                msg = (
                    f"{self.__class__.__name__} cannot process rewritten query columns {extra}. "
                    f"These columns will be ignored during retrieval, which may not be the intended behavior. "
                    f"Set on_column_mismatch='ignore' to suppress this error or 'warn' for a warning instead."
                )
                raise ValueError(msg)
            else:  # warn mode
                msg = (
                    f"{self.__class__.__name__} cannot process rewritten query columns {extra}. "
                    f"These columns will be ignored during retrieval. "
                    f"Set on_column_mismatch='ignore' to suppress this warning or 'error' to raise an error instead."
                )
            warnings.warn(msg, RuntimeWarning)

        if "docno" not in topics.columns:
            return super().transform(topics)
        elif "qid" not in topics.columns:
            raise ValueError('The dataframe needs to have a column "qid"')

        keeping = topics.columns

        common_columns = np.intersect1d(topics.columns, self.df.columns)

        # we drop columns in topics that exist in the self.df
        drop_columns = [i for i in common_columns if i not in ("qid", "docno")]
        if len(drop_columns) > 0:
            keeping = topics.columns[~topics.columns.isin(drop_columns)]

        return topics[keeping].merge(self.df, on=["qid", "docno"])


class TiraFullRankTransformer(Transformer):
    """
    A Transformer that re-executes some full-rank approach submitted to a shared task in TIRA.
    """

    def __init__(self, approach: str, tira_client: "TiraClient", input_dir, **kwargs):
        self.approach = approach
        self.tira_client = tira_client
        self.input_dir = input_dir

    def transform(self, topics):
        output_dir = tempfile.TemporaryDirectory("-pt-tira-local-execution-full-rank-transformer").name + "/output"
        os.makedirs(output_dir)

        self.tira_client.local_execution.run(
            identifier=self.approach, input_dir=self.input_dir, output_dir=output_dir, evaluate=False, dry_run=False
        )

        return merge_runs(topics, output_dir + "/run.txt")


class TiraRerankingTransformer(Transformer):
    """
    A Transformer that loads runs from TIRA that reranked some existing run.
    """

    def __init__(self, approach, tira_client, dataset=None, datasets=None, **kwargs):
        self.task, self.team, self.software = approach.split("/")
        self.tira_client = tira_client
        from tira.ir_datasets_util import translate_irds_id_to_tirex

        if dataset and datasets:
            raise ValueError(
                f"You can not pass both, dataset and datasets. Got dataset = {dataset} and datasets= {datasets}"
            )

        if not datasets and dataset:
            datasets = [dataset]
        self.datasets = datasets

        if self.datasets:
            self.datasets = [translate_irds_id_to_tirex(i) for i in self.datasets]

    def transform(self, topics):
        import numpy as np

        assert "qid" in topics.columns

        if (
            "tira_task" not in topics.columns
            or "tira_dataset" not in topics.columns
            or "tira_first_stage_run_id" not in topics.columns
        ):
            if self.datasets:
                tira_configurations = [
                    {"tira_task": self.task, "tira_dataset": i, "tira_first_stage_run_id": None} for i in self.datasets
                ]
            else:
                raise ValueError(
                    "This run needs to know the tira metadata: tira_task, tira_dataset, and tira_first_stage_run_id"
                    " needs to be in the columns of the dataframe"
                )
        else:
            tira_configurations = [
                json.loads(i)
                for i in topics[["tira_task", "tira_dataset", "tira_first_stage_run_id"]]
                .apply(lambda i: json.dumps(i.to_dict()), axis=1)
                .unique()
            ]

        df = []
        for tira_configuration in tira_configurations:
            df += [
                self.tira_client.download_run(
                    tira_configuration["tira_task"],
                    tira_configuration["tira_dataset"],
                    self.software,
                    self.team,
                    tira_configuration["tira_first_stage_run_id"],
                )
            ]
        df = pd.concat(df)
        df["qid"] = df["query"].astype(str)
        df["docno"] = df["docid"].astype(str)
        del df["query"]
        del df["docid"]

        common_columns = np.intersect1d(topics.columns, df.columns)

        # we drop columns in topics that exist in the df
        keeping = topics.columns
        drop_columns = [i for i in common_columns if i not in {"qid", "docno"}]
        if len(drop_columns) > 0:
            keeping = topics.columns[~topics.columns.isin(drop_columns)]

        join_criterium = ["qid"] if "docno" not in keeping else ["qid", "docno"]
        return topics[keeping].merge(df, how="left", left_on=join_criterium, right_on=join_criterium)


class TiraLocalExecutionRerankingTransformer(Transformer):
    """
    A Transformer that re-execues software submitted in TIRA.
    """

    def __init__(self, approach, tira_client, irds_id=None, **kwargs):
        self.task, self.team, self.software = approach.split("/")
        self.approach = approach
        self.tira_client = tira_client
        self.irds_id = irds_id

    def transform(self, topics):
        assert "qid" in topics.columns

        tmp_directory = tempfile.TemporaryDirectory("-pt-tira-local-execution-reranking-transformer").name
        input_dir = self.tira_client.pt.create_rerank_file(run_df=topics, irds_dataset_id=self.irds_id)
        output_dir = tmp_directory + "/output"
        os.makedirs(output_dir)

        self.tira_client.local_execution.run(
            identifier=self.approach, input_dir=input_dir, output_dir=output_dir, evaluate=False, dry_run=False
        )

        return merge_runs(topics, tmp_directory + "/output/run.txt")


class TiraApplyFeatureTransformer(Transformer):
    """
    A Transformer that takes a mapping of values (e.g. docids or qids) to features, and applies this
    mapping to each row of the dataframe to obtain the right feature value or vector for that row.

    The `id_cols` parameter specifies the columns that are used to identify the right feature for a row.
    For instance:
    - For query-only features, we set `id_cols` = ('qid',) and the output feature will be `mapping[row['qid']]`.
    - For document-only features, we set `id_cols` = ('docno',) and the output feature will be `mapping[row['docno']]`.
    - For query-document features, we set `id_cols` = ('qid', 'docno') and the output feature will be
        `mapping[row['qid']][row['docno']]`.
    """

    def __init__(
        self, mapping: Mapping, id_cols: "Iterable[str]" = ("qid", "docno"), name: str = "apply_features"
    ) -> None:
        self.mapping = mapping
        self.id_cols = id_cols
        self.name = name

    def __repr__(self):
        return f"tira.pt.{self.name}()"

    def transform(self, inputRes):
        outputRes = inputRes.copy()
        outputRes["features"] = outputRes.apply(self._get_features_from_row, axis=1)
        return outputRes

    def _get_features_from_row(self, row):
        value = self.mapping
        for col in self.id_cols:
            value = value[str(row[col])]

        return value



import tempfile
import unittest
from pathlib import Path

from tira.check_format import report_valid_formats

from . import (
    EMPTY_OUTPUT,
    IR_DOCUMENT_OUTPUT,
    IR_METADATA_INVALID,
    IR_METADATA_MULTIPLE_VALID,
    IR_METADATA_SINGLE_VALID,
    IR_QUERY_OUTPUT,
    TSV_OUTPUT_VALID,
    VALID_RUN_OUTPUT,
)


class TestValidFormats(unittest.TestCase):
    def test_empty_output(self):
        expected = {}
        actual = report_valid_formats(EMPTY_OUTPUT)
        self.assertEqual(expected, actual)

    def test_ir_metadata_invalid(self):
        expected = {}
        actual = report_valid_formats(IR_METADATA_INVALID)
        self.assertEqual(expected, actual)

    def test_query_output(self):
        expected = {"query-processor": True}
        actual = report_valid_formats(IR_QUERY_OUTPUT)
        self.assertEqual(expected, actual)

    def test_tsv_output(self):
        expected = {}
        actual = report_valid_formats(TSV_OUTPUT_VALID)
        self.assertEqual(expected, actual)

    def test_run_output(self):
        expected = {"run.txt": True}
        actual = report_valid_formats(VALID_RUN_OUTPUT)
        self.assertEqual(expected, actual)

    def test_document_output(self):
        expected = {"document-processor": True}
        actual = report_valid_formats(IR_DOCUMENT_OUTPUT)
        self.assertEqual(expected, actual)

    def test_pyterrier_index_output(self):
        expected = {"terrier-index": True}
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "index").mkdir(parents=True, exist_ok=True)
            (Path(d) / "index" / "data.properties").write_text("")
            (Path(d) / "index" / "data.meta.idx").write_text("")

            actual = report_valid_formats(Path(d))
            self.assertEqual(expected, actual)

    def test_ir_metadata_multiple_valid(self):
        expected = {
            "ir_metadata": [
                ".ir-metadata.yml",
                "foo-metadata.yml",
                "foo-metadata.yml",
                "m.yaml",
                "m.yaml",
                "metadata.yaml",
            ]
        }
        actual = report_valid_formats(IR_METADATA_MULTIPLE_VALID)
        self.assertEqual(expected, actual)

    def test_ir_metadata_single_valid(self):
        expected = {
            "ir_metadata": [
                ".ir-metadata.yml",
            ]
        }
        actual = report_valid_formats(IR_METADATA_SINGLE_VALID)
        self.assertEqual(expected, actual)



import gzip
import io
import json
import logging
import os
import sys
import uuid
import zipfile
from contextlib import redirect_stderr, redirect_stdout
from datetime import datetime as dt
from glob import glob
from pathlib import Path
from subprocess import check_output
from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Tuple, Union

import pandas as pd
from tqdm import tqdm

from tira.check_format import FormatMsgType, _fmt, log_message
from tira.tira_client import TiraClient


def dataset_as_dataframe(
    dataset_id_or_path: "Union[str, Path]", dataset_format: str, tira_client: "Optional[TiraClient]" = None
):
    """Load all entries in a dataset (either a local directory passed as Path or the TIRA ID of a dataset) in the specified format.

    Args:
        dataset_id_or_path (Union[str, Path]): the dataset that should be iterated, either as a path to a local dataset directory or the ID of a TIRA dataset.
        dataset_format (str): The format of the dataset.
        tira_client (TiraClient, optional): The rest API client to load the dataset in case the dataset is provided as ID. Defaults to None to use the default REST API.

    Returns:
        pd.DataFrame: The entries in the dataset parsed into a pandas DataFrame.
    """
    import pandas as pd

    ret = [i for i in dataset_as_iterator(dataset_id_or_path, dataset_format, tira_client)]
    return pd.DataFrame(ret)


def dataset_as_iterator(
    dataset_id_or_path: "Union[str, Path]", dataset_format: str, tira_client: "Optional[TiraClient]" = None
):
    """Load all entries in a dataset (either a local directory passed as Path or the TIRA ID of a dataset) in the specified format.

    Args:
        dataset_id_or_path (Union[str, Path]): the dataset that should be iterated, either as a path to a local dataset directory or the ID of a TIRA dataset.
        dataset_format (str): The format of the dataset.
        tira_client (TiraClient, optional): The rest API client to load the dataset in case the dataset is provided as ID. Defaults to None to use the default REST API.

    Yields:
        ANY: the entries in the dataset parsed in the specified format.
    """
    if Path(dataset_id_or_path).exists():
        dataset = Path(dataset_id_or_path)
    else:
        if tira_client is None:
            from tira.rest_api_client import Client

            tira_client = Client()

        dataset = Path(tira_client.download_dataset(dataset=dataset_id_or_path, task=None, allow_local_dataset=True))
    from tira.check_format import lines_if_valid

    for i in lines_if_valid(dataset, dataset_format):
        yield i


def verify_docker_installation() -> Tuple[FormatMsgType, str]:
    try:
        from tira.local_execution_integration import LocalExecutionIntegration

        local_execution: LocalExecutionIntegration = LocalExecutionIntegration()
        assert local_execution.docker_is_installed_failsave()
        return _fmt.OK, "Docker/Podman is installed."
    except:
        return _fmt.ERROR, "Docker/Podman is not installed. You can not run dockerized TIRA submissions."


def tira_home_exists():
    try:
        from tira.rest_api_client import Client

        assert Path(Client().tira_cache_dir).exists()
        return _fmt.OK, "TIRA home is writable."
    except:
        return _fmt.ERROR, "TIRA can not write data to disk, ensure that TIRA_CACHE_DIR is writable."


def api_key_is_valid():
    try:
        from tira.rest_api_client import Client

        assert Client().api_key_is_valid()
        return _fmt.OK, "You are authenticated against www.tira.io."
    except:
        return _fmt.WARN, "Your TIRA client is not authenticated. Please run 'tira-cli login'."


def verify_tirex_tracker():
    return _fmt.OK, "The tirex-tracker works and will track experimental metadata."


def verify_tira_installation() -> FormatMsgType:
    ret = _fmt.OK

    checks: List[Callable] = [api_key_is_valid, tira_home_exists, verify_docker_installation, verify_tirex_tracker]

    for i in checks:
        status, msg = i()
        log_message(msg, status)
        if status == _fmt.ERROR:
            ret = _fmt.ERROR
        if status == _fmt.WARN and ret != _fmt.ERROR:
            ret = _fmt.WARN

    return ret


def parse_jsonl_line(input: Union[str, bytearray, bytes], load_default_text: bool) -> Dict:
    """
    Deseralizes the line using JSON deserialization. Optionally strips the 'original_query' and 'original_document'
    fields from the resulting object and converts the qid and docno fields to strings.

    :param str | bytearray | bytes input: A json-serialized string.
    :param bool load_default_text: If true, the original_query and original_document fields are removed and the qid and
        docno values are converted to strings.
    :return: The deserialized and (optionally) processed object.
    :rtype: dict

    :Example:
        >>> parse_jsonl_line('{}', False)
        {}
        >>> parse_jsonl_line('{"original_query": "xxxx"}', False)
        {'original_query': 'xxxx'}
        >>> parse_jsonl_line('{"original_query": "xxxx"}', True)
        {}
        >>> parse_jsonl_line('{"original_query": "xxxx", "qid": 42, "pi": 3.14}', False)
        {'original_query': 'xxxx', 'qid': 42, 'pi': 3.14}
        >>> parse_jsonl_line('{"original_query": "xxxx", "qid": 42, "pi": 3.14}', True)
        {'qid': '42', 'pi': 3.14}
    """
    obj = json.loads(input)
    assert isinstance(obj, dict)
    if load_default_text:
        for field_to_del in ["original_query", "original_document"]:
            obj.pop(field_to_del, None)

        for field_to_str in ["qid", "docno"]:
            if field_to_str in obj:
                obj[field_to_str] = str(obj[field_to_str])

    return obj


def flush_stdout_and_stderr(closable=None):
    sys.stderr.flush()
    sys.stdout.flush()
    if closable:
        closable.close()
        print("\n")


def zip_dir(file_path: Path, allow_list: "Optional[set]" = None) -> Path:
    if os.path.isfile(file_path):
        return zip_dir(Path(file_path).parent, set([Path(file_path).name]))
    from tira.third_party_integrations import temporary_directory

    zip_file = temporary_directory()
    zip_file = zip_file / "tira-upload.zip"

    zf = zipfile.ZipFile(zip_file, "w", compression=zipfile.ZIP_DEFLATED, compresslevel=9)
    for root, _, files in os.walk(file_path):
        for name in files:
            if allow_list is not None and name not in allow_list:
                continue
            filePath = os.path.join(root, name)
            zf.write(filePath, arcname=Path(filePath).relative_to(file_path))

    zf.close()
    return zip_file


def stream_all_lines(
    input_file: Union[str, Iterable[bytes], Path], load_default_text: bool
) -> Generator[Dict, Any, Any]:
    """
    .. todo:: add documentation
    .. todo:: this function has two semantics: handling a file and handling file-contents
    """
    if isinstance(input_file, Path):
        yield from stream_all_lines(str(input_file), load_default_text)
        return

    if type(input_file) is str:
        if not os.path.isfile(input_file):
            return

        if input_file.endswith(".gz"):
            with gzip.open(input_file, "rt", encoding="utf-8") as f:
                yield from stream_all_lines(f, load_default_text)
        else:
            with open(input_file, "r") as f:
                yield from stream_all_lines(f, load_default_text)

        return

    for line in input_file:
        yield parse_jsonl_line(line, load_default_text)


def huggingface_model_mounts(models: "Iterable[str]") -> dict:
    """Determine the mounts to make the described huggingface models available in the container. The models must
    already exist in the local huggingface cache of the host.

    Args:
        models (Iterable[str]): A list of huggingface models that you want to mount, e.g., ["openai-community/gpt2"].

    Returns:
        dict: The mounts required to make the specified models available in the container.
    """
    hf_cache = Path(os.path.expanduser("~/.cache/huggingface/hub"))
    if "HF_HUB_CACHE" in os.environ:
        hf_cache = Path(os.environ["HF_HUB_CACHE"])
    elif "HF_HOME" in os.environ:
        hf_cache = Path(os.environ["HF_HOME"]) / "hub"
    ret = {}
    if not models:
        return ret

    hf_cache = hf_cache.absolute()
    for model in models:
        model_in_fs = ("models/" + str(model)).replace("/", "--")
        model_path = hf_cache / model_in_fs
        if not model_path.exists():
            msg = (
                f"Model {model} does not in HF_HUB_CACHE = '{hf_cache}'. Expected a directory '{model_path}' to exist."
                " Please ensure that the model is available via your preferred way to download it."
            )
            print(msg)
            raise ValueError(msg)
        else:
            ret[str(model_path)] = {"bind": f"/root/.cache/huggingface/hub/{model_in_fs}", "mode": "ro"}

    return ret


def change_workdir_cmd(workdir: str):
    """
    Returns a shell command that changes the working directory to the specified directory to be executed in the TIRA
    sandbox before the real command.
    """

    if not workdir:
        return 'echo "did not change the working directory"'

    workdir = workdir.strip()

    if not workdir or not workdir.startswith("/"):
        return 'echo "did not change the working directory"'

    return f'cd {workdir}; echo "changed workdir to {workdir}"'


def _ln_huggingface_model_mounts(models: str) -> str:
    """Create a set of ln statements for symbolic links to huggingface models within a running container. Fails if the
    models do not exist in the local huggingface cache of the host.

    Args:
        models (str): A space seperated list of huggingface models that you want to mount, e.g.,
            "openai-community/gpt2 openai-community/gpt2-large".

    Returns:
        str: the ln command for usage with eval.
    """
    if not models:
        return ""
    models = sorted(list(set([i for i in models.split(" ") if i])))

    if not models:
        return ""

    ret = ["mkdir -p /root/.cache/huggingface/hub/"]
    for model in models:
        i = huggingface_model_mounts([model])
        k = list(i.keys())
        if not k or not len(k) == 1:
            raise ValueError(f"Expected exactly one model to be mounted, got: {i}")
        target_dir = i[k[0]]["bind"]
        src_dir = k[0]

        ret += [f"rm -Rf {target_dir}", f"ln -s {src_dir} {target_dir}"]

    return "; ".join(ret + [f'echo "mounted {len(models)} models"'])


def all_lines_to_pandas(input_file: Union[str, Iterable[str], Path], load_default_text: bool) -> pd.DataFrame:
    """
    .. todo:: add documentation
    .. todo:: this function has two semantics: handling a file and handling file-contents
    """
    if isinstance(input_file, Path):
        return all_lines_to_pandas(str(input_file), load_default_text)
    if type(input_file) is str:
        if input_file.endswith(".gz"):
            with gzip.open(input_file, "rt", encoding="utf-8") as f:
                return all_lines_to_pandas(f, load_default_text)
        else:
            with open(input_file, "r") as f:
                return all_lines_to_pandas(f, load_default_text)

    import pandas as pd

    ret = []

    for line in input_file:
        ret += [parse_jsonl_line(line, load_default_text)]

    return pd.DataFrame(ret)


def __num(input: str) -> "Union[str, int, float]":
    """
    Converts the input to an int or float if possible. Returns the inputted string otherwise.

    :param str input: The string that should be converted to a float or int if possible.
    :return: The intrepteted input.
    :rtype: str | int | float

    :Example:
        >>> __num("hello world")
        "hello world"
        >>> __num("-42")
        -42
        >>> __num("3.5")
        3.5
        >>> __num("2e-6")
        2e-6
        >>> __num(" -42")
        " -42"
    """
    try:
        return int(input)
    except ValueError:
        try:
            return float(input)
        except ValueError:
            return input


def run_cmd(cmd: "List[Optional[str]]", ignore_failure=False):
    import subprocess

    exit_code = subprocess.call(cmd)

    if not ignore_failure and exit_code != 0:
        raise ValueError(f"Command {cmd} did exit with return code {exit_code}.")


def get_tira_id() -> str:
    return dt.now().strftime("%Y-%m-%d-%H-%M-%S")


def persist_tira_metadata_for_job(run_dir, run_id, input_run_id, software_id, dataset_id, task_id):
    with open(os.path.join(run_dir, "run.prototext"), "w") as f:
        f.write(run_prototext(run_id, input_run_id, software_id, dataset_id, task_id))

    with open(os.path.join(run_dir, "file-list.txt"), "wb") as f:
        file_list = check_output(["tree", "-ahv", os.path.join(run_dir, "output")])
        f.write(file_list)

    size_txt = create_tira_size_txt(run_dir)
    with open(os.path.join(run_dir, "size.txt"), "wb") as f:
        f.write(size_txt)


def create_tira_size_txt(run_dir):
    ret = check_output(
        ["bash", "-c", '(du -sb "' + str(run_dir.parent) + '" && du -hs "' + str(run_dir.parent) + '") | cut -f1']
    )
    ret += check_output(["bash", "-c", 'find "' + str(run_dir) + '" -type f -exec cat {} + | wc -l'])
    ret += check_output(["bash", "-c", 'find "' + str(run_dir) + '" -type f | wc -l'])
    ret += check_output(["bash", "-c", 'find "' + str(run_dir) + '" -type d | wc -l'])
    return ret


def patch_ir_metadata(run_dir: str, src_pattern: Dict, target_pattern: Dict) -> None:
    import yaml

    from tira.check_format import IrMetadataFormat

    fmt = IrMetadataFormat()
    fmt.apply_configuration_and_throw_if_invalid({})
    for i in fmt.all_lines(run_dir):
        name = i["name"]
        cnt = i["content"]

        if (
            "data" in cnt
            and "test collection" in cnt["data"]
            and "name" in cnt["data"]["test collection"]
            and cnt["data"]["test collection"]["name"] == src_pattern["data"]["test collection"]["name"]
        ):
            cnt["data"]["test collection"]["name"] = target_pattern["data"]["test collection"]["name"]
            with open(Path(run_dir) / name, "w") as f:
                yaml.safe_dump(cnt, f)


class TqdmUploadFile:
    def __init__(self, file_path: Path, desc: str) -> None:
        self.file = open(file_path, "rb")
        self.file_size = os.path.getsize(file_path)
        self.tqdm = tqdm(total=self.file_size, desc=desc, unit="B", unit_scale=True)

    def read(self, size: int = -1) -> bytes:
        chunk = self.file.read(size)
        self.tqdm.update(len(chunk))
        return chunk

    def __getattr__(self, attr: str) -> Any:
        return getattr(self.file, attr)

    def close(self):
        self.file.close()
        self.tqdm.close()


class MonitoredExecution:
    def run(self, method: Callable):
        from tira.third_party_integrations import temporary_directory

        ret = temporary_directory()
        output_dir = ret / "output"
        output_dir.mkdir(parents=True, exist_ok=True)
        stdout = io.StringIO()
        stderr = io.StringIO()
        exception_text = ""
        with redirect_stdout(stdout), redirect_stderr(stderr):
            try:
                method(output_dir)
            except Exception as e:
                exception_text = "\n\n" + str(repr(e))

        (ret / "stdout.txt").write_text(stdout.getvalue() + exception_text)
        (ret / "stderr.txt").write_text(stderr.getvalue() + exception_text)

        return ret


def run_prototext(run_id: str, input_run_id: str, software_id: str, dataset_id: str, task_id: str) -> str:
    return (
        '''softwareId: "'''
        + str(software_id)
        + '''"
runId: "'''
        + run_id
        + '''"
inputDataset: "'''
        + dataset_id
        + '''"
inputRun: "'''
        + input_run_id
        + '''"
downloadable: false
deleted: false
taskId: "'''
        + task_id
        + '''"
accessToken: "'''
        + str(uuid.uuid4())
        + '''"'''
    )


def parse_prototext_key_values(file_name):
    lines = open(file_name, "r").read()
    lines = lines.replace("measure{", "measure {")
    for i in [i for i in lines.split("measure {")]:
        ret = {}
        for line in i.split("\n"):
            if len(line.split(":")) < 2:
                continue
            elif len(line.split(":")) > 2:
                raise ValueError(f'Could not parse "{line}".')
            key = line.split(":")[0]
            value = line.split(":")[1].split('"')[1]
            ret[key.strip()] = __num(value.strip())
        if len(ret) > 0:
            yield ret


def to_prototext(m: List[Dict[str, Any]], upper_k: str = "") -> str:
    ret = ""

    def _to_prototext(d: Dict[str, Any], upper_k: str = "") -> str:
        ret = ""
        for k, v in d.items():
            new_k = upper_k
            if not new_k:
                new_k = k
            elif not new_k.endswith(k):
                new_k = upper_k + "_" + k
            if isinstance(v, dict):
                ret += _to_prototext(v, upper_k=new_k)
            else:
                ret += 'measure{\n  key: "' + str(new_k.replace("_", " ").title()) + '"\n  value: "' + str(v) + '"\n}\n'
        return ret

    for d in m:
        ret += _to_prototext(d, upper_k=upper_k)

    return ret


def all_environment_variables_for_github_action_or_fail(params):
    ret = {}

    for i in params:
        if len(i.split("=")) != 2:
            raise ValueError(f"Expect that exactly one '=' occurs in each passed parameter, got: '{i}'")

        key, value = i.split("=")

        if key in ret:
            raise ValueError(f"Got duplicated key: '{key}'")

        ret[key] = value

    expected_keys = [
        "GITHUB_SHA",
        "TIRA_VM_ID",
        "TIRA_TASK_ID",
        "TIRA_DOCKER_REGISTRY_TOKEN",
        "TIRA_DOCKER_REGISTRY_USER",
        "TIRA_CLIENT_TOKEN",
        "TIRA_CLIENT_USER",
        "TIRA_CODE_REPOSITORY_ID",
    ]

    for k in expected_keys:
        if k not in ret or not ret[k]:
            raise ValueError(
                f"I need the parameter {k} to continue, but could not find one or it is empty. This likely is a"
                " configuration error, e.g., due to missing secrets."
            )

    if "TIRA_JUPYTER_NOTEBOOK" in ret:
        for to_del in ["TIRA_DOCKER_FILE", "TIRA_DOCKER_PATH", "TIRA_COMMAND"]:
            if to_del in ret:
                del ret[to_del]
        ret["IMAGE_TAG"] = (
            f'registry.webis.de/code-research/tira/tira-user-{ret["TIRA_VM_ID"]}/submission:{ret["GITHUB_SHA"]}'
        )
    else:
        for expected_key in ["TIRA_DOCKER_FILE", "TIRA_DOCKER_PATH", "TIRA_COMMAND"]:
            if k not in ret or not ret[k]:
                raise ValueError(
                    f"I need the parameter {k} to continue, but could not find one or it is empty. This likely is a"
                    " configuration error, e.g., due to missing secrets."
                )

            submission_id = f'{ret["TIRA_DOCKER_PATH"].replace("/", "-").replace(" ", "-")}:{ret["GITHUB_SHA"]}'
            ret["IMAGE_TAG"] = (
                f'registry.webis.de/code-research/tira/tira-user-{ret["TIRA_VM_ID"]}/submission-{submission_id}'
            )

    return [k + "=" + v for k, v in ret.items()]


def extract_volume_mounts(v):
    v_modified = v.replace(":\\", "XYZ__________XYZ")
    volume_dir, volume_bind, volume_mode = v_modified.split(":")
    return volume_dir.replace("XYZ__________XYZ", ":\\"), volume_bind, volume_mode


def load_output_of_directory(directory: Path, evaluation: bool = False) -> "Union[Dict, pd.DataFrame]":
    files = glob(str(directory) + "/*")

    if evaluation:
        files = [i for i in files if i.endswith(".prototext")]

    if len(files) != 1:
        raise ValueError("Expected exactly one output file. Got: ", files)

    files = files[0]

    logging.debug(f"Read file from {files}")

    if evaluation:
        ret = {}
        for i in [i for i in open(files, "r").read().split("measure") if "key:" in i and "value:" in i]:
            key = i.split("key:")[1].split("value")[0].split('"')[1]
            value = i.split("key:")[1].split("value")[1].split('"')[1]

            ret[key.strip()] = __num(value.strip())

        return ret
    else:
        return pd.read_json(files, lines=True, orient="records")



import unittest
from pathlib import Path

from tira.rest_api_client import Client

RESOURCES = Path(__file__).parent.parent / "resources"
CACHE_DIR_WITH_DATASETS = RESOURCES / "cache-dir-with-datasets"


class DatasetMetadataTest(unittest.TestCase):
    def test_error_message_for_non_existing_dataset(self):
        expected_message = 'The dataset "does-not-exist" is not publicly available in TIRA. Please visit https://tira.io/datasets for an overview of all public datasets.'
        tira = Client()

        with self.assertRaises(Exception) as context:
            tira.get_dataset("does-not-exist")

        self.assertTrue(expected_message in str(repr(context.exception)))

    def test_ir_dataset_with_tira_id_and_wrong_task(self):
        expected_message = 'The dataset "does-not-exist/clueweb09-en-trec-web-2009-20230107-training" is not publicly available in TIRA. Please visit https://tira.io/datasets for an overview of all public datasets.'
        tira = Client()

        with self.assertRaises(Exception) as context:
            tira.get_dataset("does-not-exist/clueweb09-en-trec-web-2009-20230107-training")

        self.assertTrue(expected_message in str(repr(context.exception)))

    def test_ir_dataset_with_tira_id(self):
        tira = Client()
        expected = "clueweb09-en-trec-web-2009-20230107-training"
        actual = tira.get_dataset("clueweb09-en-trec-web-2009-20230107-training")

        self.assertIsNotNone(actual)
        self.assertEqual(expected, actual["id"])

    def test_ir_dataset_with_tira_id_and_task(self):
        tira = Client()
        expected = "clueweb09-en-trec-web-2009-20230107-training"
        actual = tira.get_dataset("ir-benchmarks/clueweb09-en-trec-web-2009-20230107-training")

        self.assertIsNotNone(actual)
        self.assertEqual(expected, actual["id"])

    def test_ir_dataset_with_ir_datasets_id_01(self):
        tira = Client()
        expected = "clueweb09-en-trec-web-2009-20230107-training"
        actual = tira.get_dataset("clueweb09/en/trec-web-2009")

        self.assertIsNotNone(actual)
        self.assertEqual(expected, actual["id"])

    def test_ir_dataset_with_ir_datasets_id_02(self):
        tira = Client()
        expected = "clueweb09-en-trec-web-2010-20230107-training"
        actual = tira.get_dataset("clueweb09/en/trec-web-2010")

        self.assertIsNotNone(actual)
        self.assertEqual(expected, actual["id"])



import json
import logging
import os
import shutil
import subprocess
import sys
import tarfile
import tempfile
import uuid
from copy import deepcopy
from pathlib import Path
from typing import TYPE_CHECKING, Optional

import docker
import pandas as pd

from tira.tirex_tracker import tirex_tracker_mounts_or_none

if TYPE_CHECKING:
    from .tira_client import TiraClient


class LocalExecutionIntegration:
    def __init__(self, tira_client=None):
        self.tira_client = tira_client

    def __normalize_command(self, cmd, evaluator):
        to_normalize = {
            "inputRun": "/tira-data/input-run",
            "outputDir": "/tira-data/output",
            "inputDataset": "/tira-data/input",
        }

        if "inputRun" in cmd and evaluator:
            to_normalize["outputDir"] = "/tira-data/eval_output"
            to_normalize["inputDataset"] = "/tira-data/input_truth"

        for k, v in to_normalize.items():
            cmd = cmd.replace("$" + k, v).replace("${" + k + "}", v)

        return cmd

    def __normalize_path(self, path):
        beheaded_path = path if not str(path).startswith("$PWD") else str(path)[4:]
        beheaded_path = (
            os.path.abspath(".")
            if len(beheaded_path) == 0 or beheaded_path in ["/", "."]
            else beheaded_path[0].replace("/", "") + beheaded_path[1:]
        )
        return beheaded_path

    def construct_verbosity_output(self, input_dir, output_dir, image, command, original_args):
        beheaded_input_dir = self.__normalize_path(input_dir)
        beheaded_output_dir = self.__normalize_path(output_dir)
        tira_run_input_dir = beheaded_input_dir if not str(beheaded_input_dir) == str(os.path.abspath(".")) else None
        tira_run_output_dir = (
            beheaded_output_dir if not str(beheaded_output_dir) == str(os.path.abspath("tira-output")) else None
        )
        tira_run_python_args = {k: v for k, v in original_args.items()}
        if type(tira_run_python_args["input_dir"]) is str and tira_run_python_args["input_dir"].startswith("$PWD"):
            tira_run_python_args["input_dir"] = self.__normalize_path(tira_run_python_args["input_dir"])
        if type(tira_run_python_args["output_dir"]) is str and tira_run_python_args["output_dir"].startswith("$PWD"):
            tira_run_python_args["output_dir"] = self.__normalize_path(tira_run_python_args["output_dir"])
        tira_run_args = [(k, v if type(v) is not str else f'"{v}"') for k, v in tira_run_python_args.items()]
        tira_run_args = [
            i.strip() for i in ["" if not tira_run_python_args[k] else f"{k}={v}" for k, v in tira_run_args] if i
        ]

        return {
            "tira-run-cli": "tira-run "
            + ("" if not tira_run_input_dir else f"--input-directory {tira_run_input_dir} ")
            + ("" if not tira_run_output_dir else f"--output-directory {tira_run_output_dir} ")
            + (
                f'--approach {original_args["identifier"]} '
                if "identifier" in original_args and original_args["identifier"] is not None
                else f'--image {image} --command \'{original_args["command"]}\''
            ),
            "tira-run-python": "tira.run(" + ", ".join(tira_run_args) + ")",
            "docker": (
                f"docker run --rm -ti -v {input_dir}:/tira-data/input:ro -v {output_dir}:/tira-data/output:rw"
                f" --entrypoint sh {image} -c '{command}'"
            ),
        }

    def build_docker_image(self, path, tag, dockerfile):
        image_build_code = subprocess.call(["docker", "build", "-f", str(dockerfile), "-t", str(tag), str(path)])

        if image_build_code != 0:
            raise ValueError(
                f"Building the docker image failed with error code {image_build_code}. See above for details."
            )

        print("\n\n Image build successfully.\n\n")

    def ensure_image_available_locally(self, image, client=None):
        try:
            output = subprocess.check_output(["docker", "images", "-q", image])
            if len(output) > 0:
                return
        except Exception:
            pass

        if client:
            try:
                if any(image in str(i) for i in client.images.list()):
                    return
            except Exception:
                pass

        print("# Pull Image\n\n")
        image_pull_code = subprocess.call(["docker", "pull", image])

        if image_pull_code != 0 and "GITHUB_ACTION" in os.environ:
            print("Skip pulling of image because everything is executed within github.")
            return

        if image_pull_code != 0:
            if client is None:
                client = self.__docker_client()
            try:
                client.images.pull(image)
                image_pull_code = 0
            except Exception:
                image_pull_code = 1

        if image_pull_code != 0:
            raise ValueError(
                f"Image could not be successfully pulled. Got return code {image_pull_code}. (expected 0.)"
            )

        print("\n\n Image pulled successfully.\n\nI will now run the software.\n\n")

    def extract_entrypoint(self, image):
        image_name = image
        docker_client = self.__docker_client()
        self.ensure_image_available_locally(image_name, docker_client)
        image = docker_client.images.get(image_name)
        ret = image.attrs["Config"]["Entrypoint"]
        if not ret:
            return None

        for i in deepcopy(ret):
            if i.startswith("[") and i.endswith("]"):
                print(i)
                ret = json.loads(i)
                break

        return self.make_command_absolute(image_name, " ".join(ret))

    def make_command_absolute(self, image_name, command):
        from tira.third_party_integrations import extract_to_be_executed_notebook_from_command_or_none

        executable = extract_to_be_executed_notebook_from_command_or_none(command)

        if not executable or executable.startswith("/") or executable.startswith("'/") or executable.startswith('"/'):
            return command
        else:
            return command.replace(
                executable,
                (self.docker_image_work_dir(image_name) + "/" + executable).replace("//", "/").replace("/./", "/"),
            )

    def __docker_linux_sockets(self):
        ret = [os.path.expanduser("~/.docker/desktop/docker.sock"), "/run/podman/podman.sock"]

        try:
            ret += ["/run/user/" + str(os.getuid()) + "/podman/podman.sock"]
        except Exception:
            pass

        return ret + ["/var/run/docker.sock"]

    def docker_is_installed_failsave(self) -> bool:
        return self.__docker_client() is not None

    def __docker_client(self) -> docker.DockerClient:
        try:
            environ = os.environ.copy()
            if sys.platform == "linux" and "DOCKER_HOST" not in environ:
                for docker_socket in self.__docker_linux_sockets():
                    if os.path.exists(docker_socket):
                        environ["DOCKER_HOST"] = "unix://" + docker_socket

                if "DOCKER_HOST" in environ:
                    logging.warn(
                        "Set DOCKER_HOST to '"
                        + environ["DOCKER_HOST"]
                        + "'. Prevent this by explicitly setting the environment variable DOCKER_HOST."
                    )

            client = docker.from_env(environment=environ)

            assert len(client.images.list()) >= 0
            assert len(client.containers.list()) >= 0
            return client
        except Exception as e:
            raise ValueError("It seems like docker is not installed?", e)

    def docker_client_is_authenticated(self, client: "Optional[TiraClient]" = None) -> bool:
        if not client:
            client = self.__docker_client()

        auth_config = client.api._auth_configs
        if (
            not self.tira_client.docker_registry()
            or "auths" not in auth_config
            or self.tira_client.docker_registry() not in auth_config["auths"]
        ):
            return False

        auth_config = auth_config["auths"][self.tira_client.docker_registry()]

        if not auth_config or "username" not in auth_config or "password" not in auth_config:
            return False

        login_response = client.login(
            username=auth_config["username"],
            password=auth_config["password"],
            registry=self.tira_client.docker_registry(),
        )

        return (
            "username" in login_response
            and "password" in login_response
            and auth_config["username"] == login_response["username"]
            and auth_config["password"] == login_response["password"]
        ) or ("Status" in login_response and "login succeeded" == login_response["Status"].lower())

    def login_docker_client(self, task_name, team_name, client=None):
        if not client:
            client = self.__docker_client()

        if self.docker_client_is_authenticated(client):
            return True

        docker_user, docker_password, docker_registry = self.tira_client.docker_credentials(task_name, team_name)

        if not docker_user or not docker_password or not docker_registry:
            print('Please login. Run "tira-cli login --token YOUR-TOKEN-HERE"')
            raise ValueError('Please login. Run "tira-cli login --token YOUR-TOKEN-HERE"')

        # print('Login: ', docker_user, docker_password, docker_registry)
        login_response = client.login(username=docker_user, password=docker_password, registry=docker_registry)

        if "Status" not in login_response or "login succeeded" != login_response["Status"].lower():
            print('Credentials are not valid, please run "tira-cli login --token YOUR-TOKEN-HERE"')
            raise ValueError(f"Login was not successfull, got: {login_response}")

        return True

    def run_and_return_tira_execution(self, task, dataset_directory, team, system_details):
        image = system_details["public_image_name"]
        command = system_details["command"]
        input_docker_software = system_details["input_docker_software"]
        docker_software_id_to_output = {}

        dataset_name = dataset_directory.split("/")[-1]
        log_file = Path(f"{self.tira_client.tira_cache_dir}/.archived/local-executions.jsonl")

        if not log_file.exists():
            with open(log_file, "w") as f:
                f.write("")

        with open(log_file, "r") as f:
            for l in f:
                try:
                    l = json.loads(l)
                    if (
                        l["dataset_directory"] == dataset_directory
                        and l["team"] == team
                        and l["task"] == task
                        and l["image"] == image
                        and l["command"] == command
                    ):
                        return l
                except:
                    pass

        if input_docker_software:
            for i in input_docker_software:
                if i["docker_software_id"] not in docker_software_id_to_output:
                    docker_software_id_to_output[i["docker_software_id"]] = self.run_and_return_tira_execution(
                        task, dataset_directory, team, i
                    )

        run_id = str(uuid.uuid4())
        output_dir = (
            Path(self.tira_client.tira_cache_dir) / "extracted_runs" / task / dataset_name / team / run_id / "output"
        )

        ret = {
            "task": task,
            "dataset": dataset_name,
            "dataset_directory": dataset_directory,
            "team": team,
            "run_id": run_id,
            "image": image,
            "command": command,
            "docker_software_id_to_output": docker_software_id_to_output,
            "output_dir": str(output_dir),
        }

        output_dir.mkdir(parents=True)
        self.run(
            image=image,
            command=command,
            input_dir=dataset_directory,
            output_dir=output_dir,
            input_run=(
                None
                if len(docker_software_id_to_output) != 1
                else list(docker_software_id_to_output.values())[0]["output_dir"]
            ),
            docker_software_id_to_output={k: v["output_dir"] for k, v in docker_software_id_to_output.items()},
        )

        if (output_dir / ".tracking-results.yml").exists():
            shutil.move(output_dir / ".tracking-results.yml", output_dir.parent / ".tracking-results.yml")

        with open(log_file, "a") as f:
            f.write(json.dumps(ret) + "\n")

        return ret

    def tirex_tracker_available_in_docker_image(self, image: str, client: "Optional[DockerClient]" = None) -> bool:
        if client is None:
            client = self.__docker_client()

        volumes = tirex_tracker_mounts_or_none()
        if not volumes:
            return False

        try:
            help_response = client.containers.run(
                image,
                entrypoint="sh",
                command="-c './tracked --help'",
                volumes=volumes,
                detach=False,
                remove=True,
                network_disabled=True,
            ).decode("UTF-8")
        except docker.errors.ContainerError:
            return False

        return "Measures runtime, energy, and many other" in help_response

    def evaluate(self, eval_dir: Path, output_dir: Path, allow_network: bool, evaluate: dict, client=None):
        if not client:
            client = self.__docker_client()

        evaluation_volumes = {str(eval_dir): {"bind": "/tira-data/eval_output", "mode": "rw"}}

        if type(evaluate) is dict and evaluate["evaluator_id"]:
            evaluation_volumes[str(evaluate["truth_directory"])] = {"bind": "/tira-data/input_truth", "mode": "ro"}
            evaluation_volumes[str(output_dir)] = {"bind": "/tira-data/input-run", "mode": "ro"}

            evaluate, image, command = (
                None,
                evaluate["evaluator_git_runner_image"],
                evaluate["evaluator_git_runner_command"],
            )
        elif type(evaluate) is not str:
            raise ValueError("ToDo Implement this case. I.e., set evaluate variable")

        if image is None or command is None:
            evaluate, image, command = self.__extract_image_and_command(evaluate, evaluator=True)

        command = self.__normalize_command(command, True)
        logging.debug(
            f"Evaluate software with: docker run --rm -ti -v {output_dir}:/tira-data/input -v"
            f" {output_dir}/:/tira-data/output --entrypoint sh {image} -c '{command}'"
        )

        container = client.containers.run(
            image,
            entrypoint="sh",
            command=f'-c "{command}; sleep .1"',
            volumes=evaluation_volumes,
            detach=True,
            remove=True,
            network_disabled=not allow_network,
        )

        for line in container.attach(stdout=True, stream=True, logs=True):
            print(line.decode("utf-8"), flush=True)

    def run(
        self,
        identifier=None,
        image=None,
        command=None,
        input_dir=None,
        output_dir=None,
        evaluate=False,
        dry_run=False,
        docker_software_id_to_output=None,
        software_id=None,
        allow_network=False,
        input_run=None,
        additional_volumes=None,
        eval_dir="tira-evaluation",
        gpu_count=0,
        cpu_count=None,
        mem_limit=None,
        gpu_device_ids=None,
    ):
        previous_stages = []
        original_args = {
            "identifier": identifier,
            "image": image,
            "command": command,
            "input_dir": input_dir,
            "output_dir": output_dir,
            "evaluate": evaluate,
            "dry_run": dry_run,
            "docker_software_id_to_output": docker_software_id_to_output,
            "software_id": software_id,
        }
        s_id = "unknown-software-id"
        if image is None or command is None:
            ds = self.tira_client.docker_software(approach=identifier, software_id=software_id)
            image, command, s_id, previous_stages = (
                ds["tira_image_name"],
                ds["command"],
                ds["id"],
                ds["ids_of_previous_stages"],
            )
        if not dry_run:
            client = self.__docker_client()

        command = self.__normalize_command(command, False)

        if not input_dir or not output_dir:
            raise ValueError("please pass input_dir and output_dir")

        input_dir = os.path.abspath(input_dir) if not str(input_dir).startswith("$PWD") else input_dir
        output_dir = os.path.abspath(output_dir) if not str(output_dir).startswith("$PWD") else output_dir

        docker_software_id_to_output = (
            {} if not docker_software_id_to_output else deepcopy(docker_software_id_to_output)
        )

        for previous_stage in previous_stages:
            if previous_stage in docker_software_id_to_output.keys():
                continue

            tmp_prev_stages = self.run(
                software_id=previous_stage,
                identifier=None,
                image=None,
                command=None,
                input_dir=input_dir,
                evaluate=False,
                dry_run=dry_run,
                output_dir=tempfile.TemporaryDirectory("-staged-execution-" + previous_stage).name + "/output",
                docker_software_id_to_output=docker_software_id_to_output,
            )
            for k, v in tmp_prev_stages.items():
                docker_software_id_to_output[k] = v

        verbose_data = self.construct_verbosity_output(input_dir, output_dir, image, command, original_args)
        logging.debug(
            f'Docker:\n\t{verbose_data["docker"]}\n\ntira-run'
            f' (python):\n\t{verbose_data["tira-run-python"]}\n\ntira-run (CLI):\n\t{verbose_data["tira-run-cli"]}\n\n'
        )

        if dry_run:
            return verbose_data

        volumes = {
            str(input_dir): {"bind": "/tira-data/input", "mode": "ro"},
            str(output_dir): {"bind": "/tira-data/output", "mode": "rw"},
        }

        if input_run:
            volumes[str(input_run)] = {"bind": "/tira-data/input-run", "mode": "ro"}

        for k, v in docker_software_id_to_output.items():
            volumes[str(os.path.abspath(v))] = {"bind": "/tira-data/input-run", "mode": "ro"}

        if additional_volumes:
            for v in additional_volumes:
                from tira.io_utils import extract_volume_mounts

                volume_dir, volume_bind, volume_mode = extract_volume_mounts(v)
                volume_dir = str(os.path.abspath(volume_dir))
                if volume_dir in volumes:
                    raise ValueError(f"Volume to mount is multiple times defined: {volume_dir}")
                volumes[volume_dir] = {"bind": volume_bind, "mode": volume_mode}

        self.ensure_image_available_locally(image, client)
        environment = {
            "outputDir": "/tira-data/output",
            "inputDataset": "/tira-data/input",
            "TIRA_DATASET_ID": "id",
            "TIRA_OUTPUT_DIR": "/tira-data/output",
            "TIRA_INPUT_DATASET": "/tira-data/input",
        }

        if input_run:
            environment["inputRun"] = "/tira-data/input-run"

        device_requests = []
        if gpu_count != 0:
            device_requests = [docker.types.DeviceRequest(count=gpu_count, capabilities=[["gpu"]])]
        elif gpu_device_ids:
            device_requests = [
                docker.types.DeviceRequest(device_ids=[str(i)], capabilities=[["gpu"]]) for i in gpu_device_ids
            ]

        entrypoint = "sh"
        entrypoint_flags = "-c"
        if self.tirex_tracker_available_in_docker_image(image, client):
            volumes.update(tirex_tracker_mounts_or_none())
            entrypoint = "/tracked"
            entrypoint_flags = "-o /tira-data/output/.tracking-results.yml -f irmetadata"

        container = client.containers.run(
            image,
            entrypoint=entrypoint,
            command=f'{entrypoint_flags} "{command}; sleep .1"',
            environment=environment,
            volumes=volumes,
            detach=True,
            remove=True,
            network_disabled=not allow_network,
            device_requests=device_requests,
            mem_limit=mem_limit,
            cpu_count=cpu_count,
            privileged=True,
        )

        for line in container.attach(stdout=True, stream=True, logs=True):
            print(line.decode("utf-8"))

        if evaluate:
            self.evaluate(eval_dir, output_dir, allow_network, client)

        if evaluate:
            approach_name = identifier if identifier else f'"{command}"@{image}'
            eval_results = {"approach": approach_name, "evaluate": evaluate}
            eval_results.update(self.load_output_of_directory(Path(eval_dir), evaluation=True))
            return self.load_output_of_directory(Path(eval_dir)), pd.DataFrame([eval_results])
        else:
            docker_software_id_to_output[s_id] = output_dir
            return docker_software_id_to_output

    def docker_image_work_dir(self, image):
        image = self.__docker_client().images.get(image).attrs["Config"]
        return "/" + image.get("WorkingDir", "")

    def export_file_from_software(self, container_path, host_path, identifier=None, image=None, software_id=None):
        """
        Export a file specified by container_path' from a software to the host_path at the host.
        """
        if image is None:
            ds = self.tira_client.docker_software(approach=identifier, software_id=software_id)
            image = ds["tira_image_name"]

        client = self.__docker_client()
        self.ensure_image_available_locally(image, client)
        docker_container = client.containers.create(image)
        strm, stat = docker_container.get_archive(container_path, None)

        with open(host_path, "wb") as f:
            for i in strm:
                f.write(i)

        tf = tarfile.open(host_path, mode="r")
        tf = tf.extractfile(stat["name"]).read()

        with open(host_path, "wb") as f:
            f.write(tf)

        docker_container.remove()

    def export_submission_from_jupyter_notebook(self, notebook):
        if not os.path.isfile(notebook):
            print(f"The notebook {notebook} does not exist. I can not continue.", file=sys.stderr)

            if "/" in notebook:
                try:
                    notebook = "/".join(notebook.split("/")[:-1])
                    print(
                        f"The directory {notebook} contains the files {os.listdir(notebook)}. Maybe you did mean one of"
                        " those?",
                        file=sys.stderr,
                    )
                except Exception:
                    pass

            return None

        ret = []

        ret += [
            "TIRA_COMMAND=/workspace/run-pyterrier-notebook.py --input ${TIRA_INPUT_DIRECTORY} --output"
            " ${TIRA_OUTPUT_DIRECTORY} --notebook /workspace/" + notebook.split("/")[-1]
        ]
        notebook_content = json.load(open(notebook, "r"))
        print(f"TODO: process content {notebook_content}.")

        return "\n".join(ret)

    def normalize_image_name(self, image, required_prefix):
        if required_prefix and not image.startswith(required_prefix):
            ret = (
                (required_prefix + "/" + (image + ":").split(":")[0][:10]).replace("//", "/")
                + ":"
                + (image.split(":")[-1] if ":" in image else "latest")
            )
            return ret.replace("-:", ":").replace("::", ":").replace("/:", ":")
        else:
            return image

    def show_docker_progress(self, line, tasks):
        from tqdm import tqdm

        if "status" not in line or (
            line["status"] != "Downloading" and line["status"] != "Extracting" and line["status"] != "Pushing"
        ):
            # skip other statuses
            return

        id = f'{line["id"]}--{line["status"]}'

        if id not in tasks:
            tasks[id] = tqdm(position=len(tasks) + 1, desc=f'{line["status"]} ({line["id"]}):')

        try:
            tasks[id].update(line["progressDetail"]["total"])
        except Exception:
            pass

    def push_image(self, image, required_prefix=None, task_name=None, team_name=None):
        client = self.__docker_client()
        if not self.docker_client_is_authenticated(client):
            self.login_docker_client(task_name, team_name, client)
        new_image = image
        if required_prefix and not image.startswith(required_prefix):
            new_image = self.normalize_image_name(image, required_prefix)
            print(f'I tag the image "{image}" as "{new_image}" for upload to TIRA (only internal).')
            client.images.get(image).tag(new_image)
            image = new_image

        tasks = {}
        push_response = ""
        for line in client.images.push(image, stream=True, decode=True):
            push_response += str(line)
            self.show_docker_progress(line, tasks)

        if "error" in push_response:
            print(push_response)
            raise ValueError("Could not push image")
        return new_image



import unittest

from tira.check_format import check_format

from . import _ERROR, _OK, AGGREGATED_RESULTS_OUTPUT_VALID, EMPTY_OUTPUT, IR_QUERY_OUTPUT, VALID_RUN_OUTPUT


class TestCheckAggregatedResultsFormats(unittest.TestCase):
    def test_invalid_validator_on_empty_output(self):
        expected = [_ERROR, "No aggregated-results.json files."]
        actual = check_format(EMPTY_OUTPUT, "aggregated-results.json")
        self.assertEqual(expected, actual)

    def test_invalid_validator_on_query_output(self):
        expected = [_ERROR, "No aggregated-results.json files."]
        actual = check_format(IR_QUERY_OUTPUT, "aggregated-results.json")
        self.assertEqual(expected, actual)

    def test_invalid_trec_run_file(self):
        expected = [_ERROR, "No aggregated-results.json files."]
        actual = check_format(VALID_RUN_OUTPUT, "aggregated-results.json")
        self.assertEqual(expected, actual)

    def test_valid_tsv(self):
        expected = [_OK, "The agregated-results.json file has the correct format."]
        actual = check_format(AGGREGATED_RESULTS_OUTPUT_VALID, "aggregated-results.json")
        self.assertEqual(expected, actual)



import unittest

from tira.check_format import check_format

from . import _ERROR, _OK, EMPTY_OUTPUT, IDEOLOGY_AND_POWER_LABELS, IDEOLOGY_AND_POWER_PREDICTIONS


class TestCheckFormatPowerAndIdeology(unittest.TestCase):
    def test_error_message_on_empty_output_for_truths(self):
        expected = "I did not find any files in the power and ideology pattern."
        actual = check_format(EMPTY_OUTPUT, "power-and-identification-truths")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_valid_output_for_truths(self):
        expected = [_OK, "I found 101 entries in the ideology and power task"]
        actual = check_format(IDEOLOGY_AND_POWER_LABELS, "power-and-identification-truths")
        self.assertEqual(expected, actual)

    def test_error_message_on_empty_output_for_predictions(self):
        expected = "I did not find any files in the power and ideology pattern."
        actual = check_format(EMPTY_OUTPUT, "power-and-identification-predictions")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn(expected, actual[1])

    def test_valid_output_for_predictions(self):
        expected = [_OK, "I found 100 entries in the ideology and power task"]
        actual = check_format(IDEOLOGY_AND_POWER_PREDICTIONS, "power-and-identification-predictions")
        self.assertEqual(expected, actual)



import unittest

from tira.check_format import check_format

from . import _ERROR, _OK, EMPTY_OUTPUT, JSONL_OUTPUT_VALID, WOWS26_COMPLETE, WOWS26_INCOMPLETE


class TestRunWithIrMetadataFormat(unittest.TestCase):
    def test_error_message_on_empty_output(self):
        actual = check_format(EMPTY_OUTPUT, "run-with-metadata")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn("I expected a file ir-metadata.yml in the directory", actual[1])

    def test_error_message_on_jsonl_output(self):
        actual = check_format(JSONL_OUTPUT_VALID, "run-with-metadata")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn("I expected a file ir-metadata.yml in the directory", actual[1])

    def test_error_message_on_incomplete_metadata(self):
        actual = check_format(WOWS26_INCOMPLETE, "run-with-metadata")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn("The required field actor.team still contains", actual[1])

    def test_ok_message_on_complete_metadata(self):
        actual = check_format(WOWS26_COMPLETE, "run-with-metadata")
        self.assertEqual(_OK, actual[0])
        self.assertIn("/complete is valid.", actual[1])
        self.assertIn("ir-metadata.yml is valid.", actual[1])



import json
import tempfile
import unittest
from pathlib import Path

from tira.check_format import check_format

from . import _ERROR, _OK, EMPTY_OUTPUT


class TestQueryProcessorFormat(unittest.TestCase):
    def test_error_message_on_empty_output(self):
        expected = [_ERROR, "The directory is no valid terrier index, it misses an index directory."]
        actual = check_format(EMPTY_OUTPUT, "terrier-index")
        self.assertEqual(expected, actual)

    def test_error_message_on_incomplete_output(self):
        expected = [_ERROR, "The directory is no valid terrier index, it misses an index/data.properties file."]
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "index").mkdir(parents=True, exist_ok=True)
            actual = check_format(Path(d), "terrier-index")
            self.assertEqual(expected, actual)

    def test_valid_output(self):
        expected = [_OK, "The directory seems to be a valid pyterrier index."]
        with tempfile.TemporaryDirectory() as d:
            (Path(d) / "index").mkdir(parents=True, exist_ok=True)
            (Path(d) / "index" / "data.properties").write_text("")
            (Path(d) / "index" / "data.meta.idx").write_text("")
            actual = check_format(Path(d), "terrier-index")
            self.assertEqual(expected, actual)



import unittest

from tira.rest_api_client import Client


class TestExportFromZenodo(unittest.TestCase):
    def test_pd_load_truths_local_cached_zip(self):
        tira = Client(tira_cache_dir="tests/resources/local_cached_zip")
        actual = tira.pd.inputs("wows-eval-foooo", "pointwise-20250309-test-foooo")
        self.assertEqual(2, len(actual))
        first_line = actual.iloc[0].to_dict()
        last_line = actual.iloc[-1].to_dict()
        self.assertEqual("e1786a4d-2b3d-4fcf-ad84-443acf3696e2", first_line["id"])
        self.assertEqual("931bbd92-c9c7-48af-b91f-9cfb7c2abdcb", last_line["id"])

    def test_pd_load_truths_local_cached_zip_02(self):
        tira = Client(tira_cache_dir="tests/resources/local_cached_zip")
        actual = tira.pd.inputs("pointwise-20250309-test-foooo")
        self.assertEqual(2, len(actual))
        first_line = actual.iloc[0].to_dict()
        last_line = actual.iloc[-1].to_dict()
        self.assertEqual("e1786a4d-2b3d-4fcf-ad84-443acf3696e2", first_line["id"])
        self.assertEqual("931bbd92-c9c7-48af-b91f-9cfb7c2abdcb", last_line["id"])

    def test_pd_load_truths_local_cached_zip_03(self):
        tira = Client(tira_cache_dir="tests/resources/local_cached_zip")
        actual = tira.pd.inputs("pointwise")
        self.assertEqual(2, len(actual))
        first_line = actual.iloc[0].to_dict()
        last_line = actual.iloc[-1].to_dict()
        self.assertEqual("e1786a4d-2b3d-4fcf-ad84-443acf3696e2", first_line["id"])
        self.assertEqual("931bbd92-c9c7-48af-b91f-9cfb7c2abdcb", last_line["id"])



import os
import tempfile
import unittest

from tira.rest_api_client import Client


class TestLocalExecutionIntegrationTest(unittest.TestCase):
    def test_pd_load_inputs_wows_re_ranking(self):
        tira = Client()
        actual = tira.pd.inputs("workshop-on-open-web-search", "re-ranking-20231027-training")
        self.assertEqual(6, len(actual))
        self.assertEqual("hubble telescope achievements", actual.iloc[0]["query"])
        self.assertEqual("The Hubble telescope discovered two moons of Pluto, Nix and Hydra.", actual.iloc[0]["text"])
        self.assertEqual(1, actual.iloc[0]["rank"])
        self.assertEqual("1", actual.iloc[0]["qid"])

    def test_pd_load_inputs_wows_re_ranking_02(self):
        tira = Client()
        actual = tira.pd.inputs("re-ranking-20231027-training")
        self.assertEqual(6, len(actual))
        self.assertEqual("hubble telescope achievements", actual.iloc[0]["query"])
        self.assertEqual("The Hubble telescope discovered two moons of Pluto, Nix and Hydra.", actual.iloc[0]["text"])
        self.assertEqual(1, actual.iloc[0]["rank"])
        self.assertEqual("1", actual.iloc[0]["qid"])

    def test_pd_load_truths_wows_re_ranking(self):
        tira = Client()
        actual = tira.pd.truths("workshop-on-open-web-search", "re-ranking-20231027-training")
        self.assertEqual(3, len(actual))
        self.assertEqual("hubble telescope achievements", actual.iloc[0]["query"])
        self.assertEqual("1", actual.iloc[0]["qid"])

    def test_pd_load_truths_wows_re_ranking_02(self):
        tira = Client()
        actual = tira.pd.truths("re-ranking-20231027-training")
        self.assertEqual(3, len(actual))
        self.assertEqual("hubble telescope achievements", actual.iloc[0]["query"])
        self.assertEqual("1", actual.iloc[0]["qid"])

    def test_pd_load_truths_local_cached_zip(self):
        tira = Client(tira_cache_dir="tests/resources/local_cached_zip")
        actual = tira.pd.truths("task-does-not-exist", "dataset-does-not-exist-20241201-training")
        self.assertEqual(13, len(actual))
        first_line = actual.iloc[0].to_dict()
        last_line = actual.iloc[12].to_dict()

        self.assertEqual("5a8865b0-19d7-4b33-bbe3-2f64ad54557f", first_line["id"])
        self.assertEqual(1051399, first_line["query_id"])
        self.assertEqual("3376628", first_line["unknown_doc_id"])
        self.assertEqual(0, first_line["qrel_unknown_doc"])

        self.assertEqual("449f69fa-df0e-4c9e-aead-61983ce9eaa8", last_line["id"])
        self.assertEqual(833860, last_line["query_id"])
        self.assertEqual("2830558", last_line["unknown_doc_id"])
        self.assertEqual(0, last_line["qrel_unknown_doc"])

    def test_pd_load_inputs_local_cached_zip(self):
        tira = Client(tira_cache_dir="tests/resources/local_cached_zip")
        actual = tira.pd.inputs("task-does-not-exist", "dataset-does-not-exist-20241201-training")
        self.assertEqual(13, len(actual))
        first_line = actual.iloc[0].to_dict()
        last_line = actual.iloc[12].to_dict()

        self.assertEqual("5a8865b0-19d7-4b33-bbe3-2f64ad54557f", first_line["id"])
        self.assertTrue("query_id" not in first_line)
        self.assertTrue("unknown_doc_id" not in first_line)
        self.assertTrue("qrel_unknown_doc" not in first_line)

        self.assertEqual("449f69fa-df0e-4c9e-aead-61983ce9eaa8", last_line["id"])
        self.assertTrue("query_id" not in last_line)
        self.assertTrue("unknown_doc_id" not in last_line)
        self.assertTrue("qrel_unknown_doc" not in last_line)

    def test_pd_load_truths_local_cached_zip_single_identifier(self):
        tira = Client(tira_cache_dir="tests/resources/local_cached_zip")
        actual = tira.pd.truths("task-does-not-exist/dataset-does-not-exist-20241201-training")
        self.assertEqual(13, len(actual))
        first_line = actual.iloc[0].to_dict()
        last_line = actual.iloc[12].to_dict()

        self.assertEqual("5a8865b0-19d7-4b33-bbe3-2f64ad54557f", first_line["id"])
        self.assertEqual(1051399, first_line["query_id"])
        self.assertEqual("3376628", first_line["unknown_doc_id"])
        self.assertEqual(0, first_line["qrel_unknown_doc"])

        self.assertEqual("449f69fa-df0e-4c9e-aead-61983ce9eaa8", last_line["id"])
        self.assertEqual(833860, last_line["query_id"])
        self.assertEqual("2830558", last_line["unknown_doc_id"])
        self.assertEqual(0, last_line["qrel_unknown_doc"])

    def test_pd_load_inputs_local_cached_zip_single_identifier(self):
        tira = Client(tira_cache_dir="tests/resources/local_cached_zip")
        actual = tira.pd.inputs("task-does-not-exist/dataset-does-not-exist-20241201-training")
        self.assertEqual(13, len(actual))
        first_line = actual.iloc[0].to_dict()
        last_line = actual.iloc[12].to_dict()

        self.assertEqual("5a8865b0-19d7-4b33-bbe3-2f64ad54557f", first_line["id"])
        self.assertTrue("query_id" not in first_line)
        self.assertTrue("unknown_doc_id" not in first_line)
        self.assertTrue("qrel_unknown_doc" not in first_line)

        self.assertEqual("449f69fa-df0e-4c9e-aead-61983ce9eaa8", last_line["id"])
        self.assertTrue("query_id" not in last_line)
        self.assertTrue("unknown_doc_id" not in last_line)
        self.assertTrue("qrel_unknown_doc" not in last_line)

    def test_pd_load_inputs_local_cached_zip_local_directory(self):
        tira = Client(tira_cache_dir="tests/resources/local_cached_zip")
        actual = tira.pd.inputs(
            "tests/resources/local_cached_zip/extracted_datasets/task-does-not-exist/dataset-does-not-exist-20241201-training/input-data/"
        )
        self.assertEqual(13, len(actual))
        first_line = actual.iloc[0].to_dict()
        last_line = actual.iloc[12].to_dict()

        self.assertEqual("5a8865b0-19d7-4b33-bbe3-2f64ad54557f", first_line["id"])
        self.assertTrue("query_id" not in first_line)
        self.assertTrue("unknown_doc_id" not in first_line)
        self.assertTrue("qrel_unknown_doc" not in first_line)

        self.assertEqual("449f69fa-df0e-4c9e-aead-61983ce9eaa8", last_line["id"])
        self.assertTrue("query_id" not in last_line)
        self.assertTrue("unknown_doc_id" not in last_line)
        self.assertTrue("qrel_unknown_doc" not in last_line)

    def test_pd_load_truths_local_cached_local_directory(self):
        tira = Client(tira_cache_dir="tests/resources/local_cached_zip")
        actual = tira.pd.truths(
            "tests/resources/local_cached_zip/extracted_datasets/task-does-not-exist/dataset-does-not-exist-20241201-training/truth-data/"
        )
        self.assertEqual(13, len(actual))
        first_line = actual.iloc[0].to_dict()
        last_line = actual.iloc[12].to_dict()

        self.assertEqual("5a8865b0-19d7-4b33-bbe3-2f64ad54557f", first_line["id"])
        self.assertEqual(1051399, first_line["query_id"])
        self.assertEqual("3376628", first_line["unknown_doc_id"])
        self.assertEqual(0, first_line["qrel_unknown_doc"])

        self.assertEqual("449f69fa-df0e-4c9e-aead-61983ce9eaa8", last_line["id"])
        self.assertEqual(833860, last_line["query_id"])
        self.assertEqual("2830558", last_line["unknown_doc_id"])
        self.assertEqual(0, last_line["qrel_unknown_doc"])

    def test_pd_load_inputs_local_from_sandbox(self):
        try:
            with tempfile.TemporaryDirectory() as tmp_dir:
                os.environ["TIRA_INPUT_DATASET"] = (
                    "tests/resources/local_cached_zip/extracted_datasets/task-does-not-exist/dataset-does-not-exist-20241201-training/input-data/"
                )
                tira = Client(tira_cache_dir=tmp_dir, base_url="fooo", archive_base_url="fooo")
                actual = tira.pd.inputs("DOES_NOT_EXIST")
                self.assertEqual(13, len(actual))
                first_line = actual.iloc[0].to_dict()
                last_line = actual.iloc[12].to_dict()

                self.assertEqual("5a8865b0-19d7-4b33-bbe3-2f64ad54557f", first_line["id"])
                self.assertTrue("query_id" not in first_line)
                self.assertTrue("unknown_doc_id" not in first_line)
                self.assertTrue("qrel_unknown_doc" not in first_line)

                self.assertEqual("449f69fa-df0e-4c9e-aead-61983ce9eaa8", last_line["id"])
                self.assertTrue("query_id" not in last_line)
                self.assertTrue("unknown_doc_id" not in last_line)
                self.assertTrue("qrel_unknown_doc" not in last_line)
        finally:
            del os.environ["TIRA_INPUT_DATASET"]



import unittest

from tira.check_format import check_format

from . import _ERROR, _OK, EMPTY_OUTPUT, IR_DATASET, TSV_OUTPUT_VALID, VALID_QREL_PATH


class TestCheckLsrCorpusFormat(unittest.TestCase):
    def test_invalid_validator_on_empty_output(self):
        actual = check_format(EMPTY_OUTPUT, "ir-dataset-corpus")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn("No ir-dataset found.", actual[1])

    def test_invalid_tsv(self):
        actual = check_format(TSV_OUTPUT_VALID, "ir-dataset-corpus")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn("No ir-dataset found.", actual[1])

    def test_invalid_qrel_file(self):
        actual = check_format(VALID_QREL_PATH, "ir-dataset-corpus")
        self.assertEqual(_ERROR, actual[0])
        self.assertIn("No ir-dataset found.", actual[1])

    def test_valid_ir_dataset(self):
        actual = check_format(IR_DATASET, "ir-dataset-corpus")
        self.assertEqual(_OK, actual[0])
        self.assertIn("Valid ir-dataset found.", actual[1])

    def test_valid_ir_dataset_01(self):
        actual = check_format(IR_DATASET, ["ir-dataset-corpus", "power-and-identification-truths"])
        self.assertEqual(_OK, actual[0], actual[1])
        self.assertIn("Valid ir-dataset found.", actual[1])

    def test_valid_ir_dataset_02(self):
        actual = check_format(
            IR_DATASET, ["ir-dataset-corpus", "power-and-identification-truths", "power-and-identification-predictions"]
        )
        self.assertEqual(_OK, actual[0], actual[1])
        self.assertIn("Valid ir-dataset found.", actual[1])
